@@ -40,8 +40,8 @@
 defines
 *****************************************************************/
 
-#define SAB_VERSION "0.51"
-#define SAB_DATE "February 19, 2001"
+#define SAB_VERSION "0.60"
+#define SAB_DATE "Jun 13, 2001"
 #define BARE_SIGN '@'
 
 #define MAX_BARE        3
@@ -442,6 +442,7 @@ char *readToBuffer(char *filename)
     length = 0xffff;
     buf = (char*) malloc(length + 1);
     count = fread(buf, 1, length + 1, f);
+    fclose(f);
     if (count > length)
         saberr("file too large (64k limit)", NULL);
     buf[count] = 0;
@@ -519,15 +520,15 @@ int main(int argc, char *argv[])
                 else
                     resultURI = arrayBare[2];
 
-                if (ecode = SablotRunProcessor(theProcessor,
+                if (0 != (ecode = SablotRunProcessor(theProcessor,
                     arrayBare[0],
                     indexBare <= 1  ? (char*)"file://stdin" : arrayBare[1],
                     resultURI,
-                    arrayParams, arrayArgs))
-                    break;
-
-                SablotGetResultArg(theProcessor, resultURI, &resultArg);
-                SablotDestroyProcessor(theProcessor);
+                    arrayParams, arrayArgs)))
+                    SablotGetResultArg(theProcessor, resultURI, &resultArg);
+                
+                if (theProcessor)
+                    SablotDestroyProcessor(theProcessor);
             }; break;
         case ID_SPS:
             {
@@ -554,7 +555,7 @@ int main(int argc, char *argv[])
         }
         
         if (ecode)
-            return 1;
+            return ecode;
         
         if (resultArg)
         {
