@@ -34,6 +34,8 @@
 #ifndef BaseHIncl
 #define BaseHIncl
 
+// GP: clean
+
 #ifdef HAVE_CONFIG_H
 #include <autocfg.h>
 #endif
@@ -49,6 +51,13 @@
 #include <assert.h>
 #include <string.h>
 
+#ifdef HAVE_ICONV_H
+#include <iconv.h>
+#else
+#undef iconv_t
+typedef void* iconv_t;
+#endif
+
 #include "platform.h"
 #include "sablot.h"
 // #include "shandler.h"
@@ -84,7 +93,10 @@ enum ListSize
 //max # of params and args passed to SablotProcessVA
 #define ARG_COUNT 32
 
+// the size of the buffer allocated from expat
 #define PARSE_BUFSIZE 16384
+// this is for the buffer with raw text to convert
+#define PARSE_CONV_BUFSIZE PARSE_BUFSIZE * 4
 
 #define EPS 1E-10
 #define PRIORITY_NOMATCH -10e6
@@ -237,6 +249,7 @@ typedef int Bool;
 #define NONZERO(x) (assert(x),x)
 #define NZ(x) NONZERO(x)
 #define cdelete(PTR) {if (PTR) delete PTR; PTR = NULL;}
+#define ccdelete(PTR, TYPED) {if (PTR) delete TYPED; PTR = NULL;}
 #define cdeleteArr(PTR) {if (PTR) delete[] PTR; PTR = NULL;}
 
 #ifdef _DEBUG
@@ -247,24 +260,31 @@ typedef int Bool;
 
 // macros for checking vertex types
 
-#define baseType(v) (NZ(v) -> vt & VT_BASE)
-#define isDaddy(v) (NZ(v) -> vt & VT_DADDY_FLAG)
+#define baseType(v) (((Vertex*)NZ(v)) -> vt & VT_BASE)
+#define basetype(v) baseType(v)
+#define isDaddy(v) (((Vertex*)NZ(v)) -> vt & VT_DADDY_FLAG)
 #define isRoot(v) (baseType(v) == VT_ROOT)
-#define isElement(v) (baseType(v) == VT_ELEMENT)
+#define isElement(v) (baseType(v) == VT_ELEMENT || baseType(v) == VT_ROOT)
 #define isText(v) (baseType(v) == VT_TEXT)
-#define isXSL(v) (NZ(v) -> vt & VT_XSL)
+#define isXSL(v) (((Vertex*)NZ(v)) -> vt & VT_XSL)
 #define isXSLElement(v) ((isElement(v)) && (isXSL(v)))
 #define isAttr(v) (baseType(v) == VT_ATTRIBUTE)
 #define isNS(v) (baseType(v) == VT_NAMESPACE)
 
 // macros for downcasting
 
-#define toNS(v) (cast(NmSpace*,v))
-#define toE(v) (cast(Element*,v))
-#define toX(v) (cast(XSLElement*,v))
-#define toA(v) (cast(Attribute*,v))
-#define toText(v) (cast(Text*,v))
-#define toD(v) (cast(Daddy*,v))
+#define toV(v) ((Vertex*)v)
+#define toNS(v) (cast(NmSpace*,toV(v)))
+#define toE(v) (cast(Element*,toV(v)))
+#define toX(v) (cast(XSLElement*,toV(v)))
+#define toA(v) (cast(Attribute*,toV(v)))
+#define toText(v) (cast(Text*,toV(v)))
+#define toD(v) (cast(Daddy*,toV(v)))
+#define toRoot(v) (cast(RootNode*,toV(v)))
+#define toComment(v) (cast(Comment*,toV(v)))
+#define toPI(v) (cast(ProcInstr*,toV(v)))
+// for "casting" a root node to the containing document
+#define toTree(v) (&(toRoot(v) -> getOwner()))
 
 #define OK FALSE
 #define NOT_OK TRUE
@@ -358,7 +378,6 @@ public:
 Memory leak test stuff
 *****************************************************************/
 #if defined(CHECK_LEAKS)
-#pragma Msg("Building with leak checking")
 #ifdef WIN32
 #define CRTDBG_MAP_ALLOC 
 #include <crtdbg.h>
