@@ -33,6 +33,8 @@
 #ifndef ContextHIncl
 #define ContextHIncl
 
+// GP: clean
+
 /*****************************************************************
 
     c   o   n   t   e   x   t   .   h
@@ -47,21 +49,25 @@ class SortDefList;
 class CList : public VertexList
 {
 public:
-    CList(Processor *proc_);
+    CList();
+    ~CList();
     virtual int compare(int i, int j);
     virtual void swap(int i, int j);
+    // overload VertexList's append and rm which set ordinals 
+    void append(Vertex *v);
+    void rm(int ndx);
     void incRefCount();
     int decRefCount();
-    eFlag sort(XSLElement *caller, Context*, SortDefList* sortDefs_ = NULL);
+    eFlag sort(Sit S, XSLElement *caller, Context*, SortDefList* sortDefs_ = NULL);
 private:
     int refCount;
     SortDefList *sortDefs;
-    eFlag makeValues(int from, int to, int level, 
+    eFlag makeValues(Sit S, int from, int to, int level, 
         XSLElement *caller, Context *ctxt);
     PList<Str*> values;
     int currLevel;
-    Processor *proc;
     int compareWithoutDocOrd(int i, int j);
+    void report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2);    
 };
 
 
@@ -77,7 +83,7 @@ class Context
 {
     friend class CList;
 public:
-    Context(Processor *proc_);
+    Context();
     ~Context();
     int 
         getPosition() const;
@@ -118,13 +124,14 @@ public:
     Bool
         contains(const Vertex *v) const;
 
-    eFlag sort(XSLElement *caller = NULL, SortDefList* sortDefs_ = NULL)
+    eFlag sort(Sit S, XSLElement *caller = NULL, SortDefList* sortDefs_ = NULL)
     {
         assert(caller || !sortDefs_);
-        E( array -> sort(caller, this, sortDefs_) );
+        E( array -> sort(S, caller, this, sortDefs_) );
         return OK;
     }
     void uniquize();
+    CList *getArrayForDOM();
 protected:
     CList 
         *array;
@@ -138,7 +145,6 @@ protected:
         position,
         virtualPosition,
         virtualSize;
-    Processor *proc;
 };
 
 #endif  //ifndef ContextHIncl
