@@ -35,15 +35,21 @@
 //
 // most constants (list sizes etc.) are in base.h
 
+// GP: clean
+
 #ifndef DataStrHIncl
 #define DataStrHIncl
 
 // error.h will be included later:
 #define BaseH_NoErrorH
 #include "base.h"
+// #include "situa.h"
 #undef BaseH_NoErrorH
 
 typedef unsigned long Phrase;
+class HashTable;
+class Tree;
+
 #define QSORT_THRESHOLD     10      // insertsort is used for smaller lists
 
 // List is a basic structure for ordered lists. 
@@ -74,6 +80,7 @@ public:
     inline T& last() const;
         // swap two elements with given indices
     virtual void swap(int,int);
+    void insertBefore(T newMember, int refIndex);
     ~List(void);
 protected:
     void grow();
@@ -100,7 +107,7 @@ public:
     void freelast(Bool);
         // free and remove all pointers
     void freeall(Bool);
-        // free and remove all pointers
+        // free and remove the given pointer
     void freerm(int, Bool);
 };
 
@@ -329,7 +336,7 @@ public:
         // find the index of the element with the given key
     int findNum(const Str &_key) const;
         // find the value of the element with the given key
-    Str* getUri(const Str &) const;
+    const Str* getUri(const Str &) const;
     Bool isHidden(const Str &) const;
     void appendConstruct(const Str &prefix, const Str& uri, Bool hide = FALSE);
 };
@@ -340,29 +347,68 @@ Q N a m e
 // container for a QName (e.g. "books:isbn") consisting
 // of the "namespace prefix", "namespace URI" and the "local part"
 
-class NSList;
-class Processor;
-
-class QName /* : public SabObj */
+class QName
 {
     friend class TreeConstructer;
 public:
-    QName(Processor *proc_);
+    QName();
     QName(const QName &other);
         // set the QName from the string received from expat (using NS_SEP)
+    QName& operator =(const QName& other);
     Bool isEmpty() const;
     void empty();
 
-        // set the name to "prefix:local". Params give ns decls and
-        // say whether to expand the default namespace
-    eFlag setLogical(const Str&, const NSList *, Bool);
         // set just the prefix
+    void setPrefix(Phrase);
+    void setLocal(Phrase);
+    void setUri(Phrase);
+
+        // return the full name
+    // Str getname(Tree& t) const;
+        // return the local part
+    Phrase getLocal() const;
+        // return the URI
+    Phrase getUri() const;
+        // return the prefix
+    Phrase getPrefix() const;
+        // return true if a prefix is defined
+    Bool hasPrefix() const;
+
+        // check for equality to another qname
+    Bool operator==(const QName &) const;
+
+        // output to a string
+    // void speak(DStr&, SpeakMode) const;
+        // check if our own prefix is bound to the given namespace
+    // Bool prefixIsNS(const Str &s) const;
+        // get the associated dictionary
+private:
+    Phrase
+        prefix,
+        uri,
+        local;
+};
+
+/****************************************
+E Q N a m e
+****************************************/
+
+class EQName
+{
+//    friend class TreeConstructer;
+public:
+    EQName();
+    EQName(const EQName&);
+    Bool isEmpty() const;
+    void empty();
+
+    void set(const QName&, const HashTable&);
     void setPrefix(const Str&);
     void setLocal(const Str&);
     void setUri(const Str&);
 
         // return the full name
-    Str getname() const;
+    void getname(Str& fullName) const;
         // return the local part
     const Str& getLocal() const;
         // return the URI
@@ -370,27 +416,20 @@ public:
         // return the prefix
     const Str& getPrefix() const;
         // return true if a prefix is defined
-    const Bool hasPrefix() const;
+    Bool hasPrefix() const;
 
         // check for equality to another qname
-    Bool operator==(const QName &) const;
+    Bool operator==(const EQName &) const;
 
-        // output to a string
-    void speak(DStr&, SpeakMode) const;
-        // check if our own prefix is bound to the given namespace
-    Bool prefixIsNS(const Str &s) const;
-        // find a prefix for this URI in the list of ns decls
-    void findPrefix(const NSList&);
-
-    Phrase
+private:
+    Str
         prefix,
         uri,
         local;
-private:
-    Processor *proc;
-
 };
 
+
+
 /*****************************************************************
     S t r S t r L i s t
 *****************************************************************/
@@ -435,39 +474,36 @@ public:
 
 
 /*****************************************************************
-    Q N a m e L i s t
+    E Q N a m e L i s t
 *****************************************************************/
 
-class QNameList : public PList<QName *>
+class EQNameList : public PList<EQName *>
 {
 public:
-    const QName* find(const QName &what) const;
+    const EQName* find(const EQName &what) const;
 };
 
 /*****************************************************************
-    Q N a m e S t r L i s t
+    E Q N a m e S t r L i s t
 *****************************************************************/
-class SituationObj;
-
-struct QNameStr
+struct EQNameStr
 {
-    QNameStr ( Processor *proc_ ): key ( proc_ )
+    EQNameStr(const EQName& key_, const Str& value_) : key(key_), value(value_)
     {
-    }
-    QName key;
+    };
+    EQName key;
     Str value;
 };
 
-class QNameStrList : public PList<QNameStr *>
+class EQNameStrList : public PList<EQNameStr *>
 {
 public:
-    QNameStrList (Processor *proc_) : proc(proc_)
+    EQNameStrList()
     {
-    }
-    const Str* find(const QName &what) const;
-    void appendConstruct(const QName &key, const Str& value);
-protected:
-    Processor *proc;
+    };
+    int findNdx(const EQName &what) const;
+    const Str* find(const EQName &what) const;
+    void appendConstruct(const EQName &key, const Str& value);
 };
 
 /*****************************************************************
@@ -672,6 +708,15 @@ Bool List<T>::isEmpty() const
     return (Bool) (!nItems);
 }
 
+template <class T>
+void List<T>::insertBefore(T newMember, int refIndex)
+{
+    append(newMember);
+    memmove(block + refIndex + 1, block + refIndex, 
+        (number()-refIndex-1) * sizeof(T));
+	block[refIndex] = newMember;
+}
+
 /*================================================================
 PList methods
 ================================================================*/
@@ -688,24 +733,24 @@ PList<T>::PList(int logBlocksize_)
 template <class T>
 void PList<T>::freelast(Bool asArray)
 {
-    deleteScalarOrVector(List<T>::last(), asArray);
-    List<T>::deppend();
+    deleteScalarOrVector(this->last(), asArray);
+    this->deppend();
 }
 
 template <class T>
 void PList<T>::freeall(Bool asArray)
 {
-    for (int i = 0; i < List<T>::nItems; i++)
-        deleteScalarOrVector(List<T>::block[i], asArray);
-    List<T>::deppendall();
+    for (int i = 0; i < this->nItems; i++)
+        deleteScalarOrVector(this->block[i], asArray);
+    this->deppendall();
 }
 
 template <class T>
 void PList<T>::freerm(int n, Bool asArray)
 {
-    assert((n >= 0) && (n < List<T>::nItems));
-    deleteScalarOrVector(List<T>::block[n], asArray);
-    List<T>::rm(n);
+    assert((n >= 0) && (n < this->nItems));
+    deleteScalarOrVector(this->block[n], asArray);
+    this->rm(n);
 }
 
 /*================================================================
@@ -722,15 +767,15 @@ template <class T>
 void SList<T>::insert(T what)
 {
     append(what);
-    int whatndx = List<T>::number() - 1;
+    int whatndx = this->number() - 1;
     int i, j;
     for (i=0; i < whatndx; i++)
         if (compare(whatndx,i) == -1) break;
     if (i < whatndx)
     {
         for (j = whatndx; j > i; j--)
-            List<T>::operator[](j) = List<T>::operator[](j-1);
-        List<T>::operator[](i) = what;
+            this->operator[](j) = this->operator[](j-1);
+        this->operator[](i) = what;
     };
 };
 
@@ -741,15 +786,15 @@ void SList<T>::insertsort(int bottom, int top)
     T curr;
     for (ceiling = bottom + 1; ceiling <= top; ceiling++)
     {
-        curr = List<T>::block[ceiling];
+        curr = this->block[ceiling];
         /* was:
         for (k = ceiling - 1; k >= bottom && compare(k, ceiling) > 0; k--)
-            List<T>::block[k+1] = List<T>::block[k];
-        List<T>::block[k+1] = curr;
+            this->block[k+1] = this->block[k];
+        this->block[k+1] = curr;
          * but need to use swap() to drag extra pointers along
          */
         for (k = ceiling - 1; k >= bottom && compare(k, k + 1) > 0; k--)
-            swap(k, k+1); 
+            this->swap(k, k+1); 
     }
 }
 
@@ -757,10 +802,10 @@ template <class T>
 void SList<T>::qsPartition(int& bottom, int& top)
 {
     int middle = (bottom + top) >> 1;
-    if (compare(bottom, top) > 0) swap(bottom, top);
-    if (compare(middle, top) > 0) swap(middle, top);
-    if (compare(bottom, middle) > 0) swap(bottom, middle);
-    // T pivot = List<T>::block[middle];
+    if (compare(bottom, top) > 0) this->swap(bottom, top);
+    if (compare(middle, top) > 0) this->swap(middle, top);
+    if (compare(bottom, middle) > 0) this->swap(bottom, middle);
+    // T pivot = this->block[middle];
     while (bottom <= top)
     {
         // boundary conditions fixed by Tim
@@ -773,7 +818,7 @@ void SList<T>::qsPartition(int& bottom, int& top)
                 middle = top;
             else if (middle == top)
                 middle = bottom;
-            swap(bottom, top);
+            this->swap(bottom, top);
         }
     }
 }
@@ -798,10 +843,11 @@ void SList<T>::sort(int from /* = 0 */, int to /* = - 1 */)
 {
     /* cannot initialize 'to' to nItems-1 directly as g++ reports error */ 
     if (to == -1)
-        to = List<T>::nItems - 1;
-    if (List<T>::nItems < 2)
+        to = this->nItems - 1;
+    if (this->nItems < 2)
         return;
     quicksort(from, to);
 }
 
 #endif //ifndef DataStrHIncl
+
