@@ -35,6 +35,8 @@
 #ifndef ExprHIncl
 #define ExprHIncl
 
+// GP: clean
+
 #include "base.h"
 #include "datastr.h"
 #include "proc.h"
@@ -106,27 +108,26 @@ public:
 //ExNodeType is defined in base.h
 //array exNodeTypeNames in base.cpp
 
-class Tokenizer /* : public SabObj */
+class Tokenizer
 {
 public:
-	int findTop(ExToken token, int from);
-    Tokenizer(Processor* proc);
+    Tokenizer(Expression &owner_);
     ~Tokenizer();
-    eFlag tokenize(const Str &astring);
-    eFlag getToken(char *&, TokenItem&, ExToken);
-    void speak(DStr &s, SpeakMode mode);
-    eFlag getDelim(int &, Bool = FALSE);
-    void stripParens(int &, int &);
+	int findTop(ExToken token, int from);
+    eFlag tokenize(Sit S, const Str &astring);
+    eFlag getToken(Sit S, char *&, TokenItem&, ExToken);
+    eFlag getDelim(Sit S, int &, Bool = FALSE);
+    eFlag stripParens(Sit S, int &, int &);
     DStr string;
-    Processor* proc;
-    SituationObj* situation;
     PList<TokenItem*> items;
 private:
     ExToken tryShort(char *&, ExToken);
-    eFlag lookToken(ExToken &, char*, ExToken);
-    eFlag getToken_(ExToken &, char*&, ExToken);
-    eFlag getNumber(char *&);
-    eFlag getName(ExToken&, char *&, ExToken);
+    eFlag lookToken(Sit S, ExToken &, char*, ExToken);
+    eFlag getToken_(Sit S, ExToken &, char*&, ExToken);
+    eFlag getNumber(Sit S, char *&);
+    eFlag getName(Sit S, ExToken&, char *&, ExToken);
+    void report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2);
+    Expression &owner;
 };
 
 /*
@@ -282,22 +283,24 @@ class LocStep /* : public SabObj */
 {
     friend class Expression;
 public:
-    LocStep(Processor* proc, ExAxis = AXIS_NONE, ExNodeType = EXNODE_NONE);
+    LocStep(Element &ownerV_, ExAxis = AXIS_NONE, ExNodeType = EXNODE_NONE);
     ~LocStep();
     void set(ExAxis, ExNodeType);
-    eFlag parse(Tokenizer&, int&, Element *ownerV);
+    eFlag parse(Sit S, Tokenizer&, int&);
     ExAxis ax;
     ExNodeType ntype;
     QName ntest;
     ExprList preds;
-    void speak(DStr &s, SpeakMode mode);
-    Bool matchesWithoutPreds(Vertex *v);
+    void speak(Sit S, DStr &s, SpeakMode mode);
+    Bool matchesWithoutPreds(Sit S, Vertex *v);
     // eFlag createContextNoPreds(Context *&, int);
-    eFlag shift(Vertex *&v, Vertex *baseV);
-    Processor* proc;
+    eFlag shift(Sit S, Vertex *&v, Vertex *baseV);
     Bool positional;
+    void report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2);
+    Element &getOwnerElement() {return ownerV;}
 private:
     int badPreds;
+    Element &ownerV;
 };
 
 
@@ -310,31 +313,27 @@ class Element;
 class Expression
 {
 public:
-    Expression(Element *, Processor* proc, ExFunctor = EXF_NONE);
+    Expression(Element & owner_, ExFunctor = EXF_NONE);
     ~Expression();
     void clearContent();
-    void
-        setLS(ExAxis, ExNodeType);
-    void speak(DStr &,SpeakMode);
-    eFlag 
-        parse(const DStr &, Bool = FALSE);
-    eFlag 
-        parse(Tokenizer&, int, int);
-    eFlag eval(Expression &, Context *);
-    Number tonumber();
-    Str tostring();
-    char *tostringCharPtr();
+    void setLS(ExAxis, ExNodeType);
+    eFlag speak(Sit S, DStr &,SpeakMode);
+    eFlag parse(Sit S, const DStr &, Bool = FALSE);
+    eFlag parse(Sit S, Tokenizer&, int, int);
+    eFlag eval(Sit S, Expression &, Context *);
+    Number tonumber(Sit S);
+    eFlag tostring(Sit S, Str& result);
     const Str& tostringRef() const;
     Bool tobool();
     Context& tonodeset();
     const Context& tonodesetRef();
-    eFlag matchesPattern(Context *, Bool &result);
-    eFlag trueFor(Context *, Bool&);
+    eFlag matchesPattern(Sit S, Context *, Bool &result);
+    eFlag trueFor(Sit S, Context *, Bool&);
     //
     ExType type;
     ExFunctor functor;
     ExprList args;
-    eFlag createContext(Context *&, int = -1);
+    eFlag createContext(Sit S, Context *&, int = -1);
     LocStep *step;
     int hasPath;
     void setAtom(Context*);
@@ -343,8 +342,6 @@ public:
     void setAtom(Bool);
     Tree *setFragment();
     Tree *pTree;
-    Element *ownerV;
-    Processor* proc;
 
     // optimizePositional() sets usesLast and positional
     // returns: 2 if pred contains last()   ( -> usesLast && positional )
@@ -358,6 +355,10 @@ public:
     // inBounds: 
     // returns 0 if the position is within bounds, -1 if before, 1 if after
     int inBounds(int position) const;
+    Element& getOwnerElement() const;
+    Tree& getOwnerTree() const;
+    void report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2);
+protected: 
 private:
     union
     {
@@ -367,25 +368,25 @@ private:
         Number *patomnumber;
         Context *patomnodeset;
     };
-    Bool compareCC(ExFunctor, const Context &, const Context &);
-    Bool compareCS(ExFunctor, const Context &, const DStr &);
-    Bool compareCN(ExFunctor, const Context &, const Number &);
-    eFlag compare(Bool &, Expression &, ExFunctor);
+    Bool compareCC(Sit S, ExFunctor, const Context &, const Context &);
+    Bool compareCS(Sit S, ExFunctor, const Context &, const DStr &);
+    Bool compareCN(Sit S, ExFunctor, const Context &, const Number &);
+    eFlag compare(Sit S, Bool &, Expression &, ExFunctor);
     Bool isOp(ExToken, int &);
-    eFlag callOp(Expression&, ExprList&);
-    eFlag callFunc(Expression&, ExprList&, Context *);
-    eFlag parseBasic(Tokenizer &, int, int);
-    eFlag parseLP(Tokenizer&, int &, Bool = FALSE);
-    eFlag matchesSinglePath(Vertex *v, int lastIndex, Bool &result);
-    eFlag matchesSingleStep(Vertex *v, Bool &result);
-    eFlag createLPContextLevel(int stepLevel, int stepsCount,
+    eFlag callOp(Sit S, Expression&, ExprList&);
+    eFlag callFunc(Sit S, Expression&, ExprList&, Context *);
+    eFlag parseBasic(Sit S, Tokenizer &, int, int);
+    eFlag parseLP(Sit S, Tokenizer&, int &, Bool = FALSE);
+    eFlag matchesSinglePath(Sit S, Vertex *v, int lastIndex, Bool &result);
+    eFlag matchesSingleStep(Sit S, Vertex *v, Bool &result);
+    eFlag createLPContextLevel(Sit S, int stepLevel, int stepsCount,
         Vertex *base, Context& info, Context *theResult);
-    eFlag createLPContext(Context *&, int, Vertex *globalCurrent = NULL);
-    eFlag createLPContextSum(Context *&, Vertex *globalCurrent = NULL);
+    eFlag createLPContext(Sit S, Context *&, int, Vertex *globalCurrent = NULL);
+    eFlag createLPContextSum(Sit S, Context *&, Vertex *globalCurrent = NULL);
 
     Bool 
         isPattern;
-    eFlag patternOK();
+    eFlag patternOK(Sit S);
     void setAtomNoCopy(Context &);
 
     // in a predicate, these give the bounds to context positions that
@@ -396,6 +397,8 @@ private:
     // in a predicate, this is true iff the predicate depends on last()
     Bool usesLast,
         positional;
+	
+	Element &owner;
 };
 
 #endif //ifndef ExprHIncl
