@@ -33,8 +33,11 @@
 #if !defined(OutputHIncl)
 #define OutputHIncl
 
+// GP: clean
+
 #include "datastr.h"
 #include "utf8.h"
+#include "encoding.h"
 
 class DataLine;     // see uri.h
 
@@ -78,28 +81,32 @@ enum EscMode
 class OutputDefinition
 {
 public:
-    OutputDefinition(Processor *proc_);
+    OutputDefinition();
     ~OutputDefinition();
-    eFlag setItemStrCheck(XSL_ATT itemId, const Str& value);
-    eFlag setItemStrIfNot(XSL_ATT itemId, const Str& value);
-    eFlag setItemStrForce(XSL_ATT itemId, const Str& value);
-    eFlag setDefaults();
-    eFlag setItemQName(XSL_ATT itemId, const QName& value, Bool doCheck);
+    eFlag setItemStrCheck(
+        Sit S, XSL_ATT itemId, const Str& value, Vertex *caller);
+    eFlag setItemStrIfNot(
+        Sit S, XSL_ATT itemId, const Str& value, Vertex *caller);
+    eFlag setItemStrForce(
+        Sit S, XSL_ATT itemId, const Str& value, Vertex *caller);
+    eFlag setDefaults(Sit S);
+    eFlag setItemEQName(Sit S, XSL_ATT itemId, const EQName& value, Bool doCheck, Vertex *caller);
     const Str& getValueStr(XSL_ATT itemId) const;
-    const QName& getValueQName(XSL_ATT itemId) const;
-    Bool askQNameList(XSL_ATT itemId, const QName &what) const;
+    const EQName& getValueEQName(XSL_ATT itemId) const;
+    Bool askEQNameList(XSL_ATT itemId, const EQName &what) const;
     int getStatus(XSL_ATT itemId) const;
     OutputMethod getMethod() const;
-    Encoding getEncoding() const;
+    Bool getIndent() const;
+    const Str& getEncoding() const;
 private:
-    eFlag setItemStr_(XSL_ATT itemId, const Str& value, Bool doCheck, Bool soft);
+    eFlag setItemStr_(Sit S, XSL_ATT itemId, const Str& value, Bool doCheck, Bool soft, Vertex *caller);
     Str stringItems[STRING_ITEMS_COUNT];
-    QName
+    EQName
         method;
-    QNameList
+    EQNameList
         cdataElems;
     DataLine *targetDataLine;
-    Processor *proc;
+    void report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2);
 };
 
 //
@@ -111,34 +118,36 @@ private:
 class PhysicalOutputLayerObj
 {
 public:
-    PhysicalOutputLayerObj(Processor *proc_);
+    PhysicalOutputLayerObj();
     ~PhysicalOutputLayerObj();
-    eFlag setOptions(DataLine *targetDataLine_, OutputDefinition *outDef_);
-    eFlag outputElementStart(const Str& name, 
+    eFlag setOptions(Sit S, DataLine *targetDataLine_, OutputDefinition *outDef_);
+    eFlag outputElementStart(Sit S, const Str& name, 
         const NamespaceStack& namespaces, const int namespace_index,
-        const QNameStrList& atts, Bool isEmpty);
-    eFlag outputElementEnd(const Str& name, Bool isEmpty);
-    eFlag outputText(const Str& contents, Bool outputEscaping, Bool inHTMLSpecial);
-    eFlag outputComment(const Str& contents);
-    eFlag outputCDataSection(const Str& contents);
-    eFlag outputPI(const Str& target, const Str& data);
-    eFlag outputDTD(const Str& name, const Str& publicId, const Str& systemId);
-    eFlag outputDone();
-    eFlag setMethodByDefault(OutputMethod method_);
+        const EQNameStrList& atts, Bool isEmpty);
+    eFlag outputElementEnd(Sit S, const Str& name, Bool isEmpty);
+    eFlag outputText(Sit S, const Str& contents, Bool outputEscaping, Bool inHTMLSpecial);
+    eFlag outputComment(Sit S, const Str& contents);
+    eFlag outputCDataSection(Sit S, const Str& contents);
+    eFlag outputPI(Sit S, const Str& target, const Str& data);
+    eFlag outputDTD(Sit S, const Str& name, const Str& publicId, const Str& systemId);
+    eFlag outputDone(Sit S);
+    eFlag setMethodByDefault(Sit S, OutputMethod method_);
 private:
-    Processor *proc;
     DataLine *targetDataLine;
     OutputDefinition *outDef;
     OutputMethod method;
+    Bool indent;
     char buffer[OUTPUT_BUFFER_SIZE],
         smallBuf[SMALL_BUFFER_SIZE];
     int curr;
-    Encoding enc;
-    eFlag sendOut(const char* data, int length, EscMode escapeMode);
-    eFlag sendOutUntil(const char* & data, int length,
+    Str encoding;
+    CDesc encodingCD;
+    eFlag sendOut(Sit S, const char* data, int length, EscMode escapeMode);
+    eFlag sendOutUntil(Sit S, const char* & data, int length,
         EscMode escapeMode, const char* stoppingText);
     int writeCharacterRef(char* dest, const char *src, EscMode escapeMode);
-    eFlag flushBuffer();
+    eFlag flushBuffer(Sit S);
+    void report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2);
 };
 
 //
@@ -188,36 +197,37 @@ struct FrontMatterItem
 class FrontMatter : public PList<FrontMatterItem*>
 {
 public:
-    SituationObj* situation;
-    eFlag appendConstruct(FrontMatterKind kind, const Str& string1,
+    eFlag appendConstruct(Sit S, FrontMatterKind kind, const Str& string1,
         const Str& string2, Bool disableEsc);
+    void report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2);
 };
 
 class OutputterObj
 {
 public:
-    OutputterObj(Processor *proc_);
+    OutputterObj();
     ~OutputterObj();
-    eFlag setOptions(DataLine *targetDataLine_, OutputDefinition *outDef_);
-    eFlag setOptionsSAX(SAXHandler *streaming_, void *userData);
-    eFlag eventBeginOutput();
-    eFlag eventElementStart(const QName& name);
-    eFlag eventElementEnd(const QName& name);
-    eFlag eventAttributeStart(const QName& name);
-    eFlag eventAttributeEnd();
-    eFlag eventCommentStart();
-    eFlag eventCommentEnd();
-    eFlag eventNamespace(const Str& prefix, const Str& uri);
-    eFlag eventPIStart(const Str& name);
-    eFlag eventPIEnd();
-    eFlag eventData(const Str& data);
-    eFlag eventDisableEscapingForNext();
-    eFlag eventEndOutput();
+    eFlag setOptions(Sit S, DataLine *targetDataLine_, OutputDefinition *outDef_);
+    eFlag setOptionsSAX(Sit S, SAXHandler *streaming_, void *userData);
+    eFlag eventBeginOutput(Sit S);
+    eFlag eventElementStart(Sit S, const EQName& name);
+    eFlag eventElementEnd(Sit S, const EQName& name);
+    eFlag eventAttributeStart(Sit S, const EQName& name);
+    eFlag eventAttributeEnd(Sit S);
+    eFlag eventCommentStart(Sit S);
+    eFlag eventCommentEnd(Sit S);
+    eFlag eventNamespace(Sit S, const Str& prefix, const Str& uri);
+    eFlag eventPIStart(Sit S, const Str& name);
+    eFlag eventPIEnd(Sit S);
+    eFlag eventData(Sit S, const Str& data, Bool hardCData = FALSE);
+    eFlag eventCDataSection(Sit S, const Str& data);
+    eFlag eventDisableEscapingForNext(Sit S);
+    eFlag eventEndOutput(Sit S);
 private:
-    eFlag reportCurrData();
-    eFlag reportStartTag(Bool isEmpty);
-    eFlag throwInMeta();
-    void pushLevel(const QName& name);
+    eFlag reportCurrData(Sit S, Bool hardCData = FALSE);
+    eFlag reportStartTag(Sit S, Bool isEmpty);
+    eFlag throwInMeta(Sit S);
+    void pushLevel(const EQName& name);
 //    void popLevel();
     int getLevel() 
     {
@@ -231,9 +241,10 @@ private:
     {
         return (history.number()? history.last() -> firstOwnNS : 0);
     }
-    eFlag reportXMLDeclIfMust();
-    eFlag reportDTDIfMust(const QName& docElementName);
-    eFlag reportFront();
+    eFlag reportXMLDeclIfMust(Sit S);
+    eFlag reportDTDIfMust(Sit S, const EQName& docElementName);
+    eFlag reportFront(Sit S);
+    void report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2);    
     //
     PhysicalOutputLayerObj *physical;
     SAXHandler *mySAXHandler;
@@ -245,18 +256,18 @@ private:
     DStr currData;
     Str
         currPIName;
-    QName
+    EQName
         currElement, 
         currAttName;
     NamespaceStack
         currNamespaces;
-    QNameStrList
+    EQNameStrList
         currAtts;
     OutputHistory history;
     Bool noElementYet,
         noHeadYet;
     FrontMatter front;
-    Processor *proc;
+
 };
 
 #endif  // OutputHIncl
