@@ -37,10 +37,9 @@
 #include "tree.h"
 #include "error.h"
 #include "proc.h"
+#include "encoding.h"
 
-// for the windows-1250 encoding:
-#include "enc1250.h"
-// (will change to encodings.h)
+// GP: clean
 
 // 
 //
@@ -61,43 +60,39 @@ SAXHandler TreeConstructer::myHandlerRecord =
     tcEndDocument
 };
 
-TreeConstructer::TreeConstructer(Processor *theProcessor_)
-:
-theProcessor( NZ(theProcessor_) )
+TreeConstructer::TreeConstructer(Sit S_)
+: S(S_)
 {
     theParser = NULL;
     theTree = NULL;
     theDataLine = NULL;
     theLineNumber = 0;
-    theSituation = theProcessor_->situation;
-    theOutputter = NULL;
 }
 
 TreeConstructer::~TreeConstructer()
 {
-    cdelete(theOutputter);
 }
 
 
-eFlag TreeConstructer::parseDataLineUsingGivenExpat(
+eFlag TreeConstructer::parseDataLineUsingGivenExpat(Sit S, 
     Tree *t, DataLine *d, XML_Parser theParser_)
 {
     theTree = t;
     theDataLine = d;
     theParser = theParser_;
-    E( feedDocumentToParser(this) );
-    E( t -> parseFinished() );
+    E( feedDocumentToParser(S, this) );
+    E( t -> parseFinished(S) );
     if (t -> XSLTree)
-        t -> stripped += t -> root.strip();
+        t -> stripped += t -> getRoot().strip();
     return OK;
 };
 
 
-eFlag TreeConstructer::parseDataLineUsingExpat(
+eFlag TreeConstructer::parseDataLineUsingExpat(Sit S, 
     Tree *t, DataLine *d)
 {
     theParser = XML_ParserCreateNS(NULL, THE_NAMESPACE_SEPARATOR);
-    M( theSituation, theParser );
+    M( S, theParser );
     // XML_UseParserAsHandlerArg(parser);
     XML_SetElementHandler(theParser, 
         tcStartElement,
@@ -111,84 +106,144 @@ eFlag TreeConstructer::parseDataLineUsingExpat(
         tcComment);
     XML_SetProcessingInstructionHandler(theParser,
         tcPI);
-    XML_SetUnknownEncodingHandler(theParser, 
-        tcUnknownEncoding, 
-        NULL);
+    // the unknown encoding handler is no more used:
+    // XML_SetUnknownEncodingHandler(theParser, 
+    //     tcUnknownEncoding, 
+    //     NULL);
+
     XML_SetExternalEntityRefHandler(theParser,
         tcExternalEntityRef);
     XML_SetUserData(theParser, this);
-    XML_SetBase(theParser, theProcessor -> findBaseURI( t -> name ));
+    if (S.getProcessor())
+        XML_SetBase(theParser, S.getProcessor() -> findBaseURI( t -> name ));
 
-    E( parseDataLineUsingGivenExpat(t, d, theParser) );
+    eFlag eCode = parseDataLineUsingGivenExpat(S, t, d, theParser);
 
     XML_ParserFree(theParser);
+    E( eCode );
     return OK;
 };
 
-eFlag TreeConstructer::parseUsingSAX(Tree *t)
+eFlag TreeConstructer::parseUsingSAX(Sit S, Tree *t, OutputterObj& source)
 {
     theTree = t;
     theDataLine = NULL;
-    M( theSituation, theOutputter = new OutputterObj(theProcessor) );
     // register the handler with the outputter
-    E( theOutputter -> setOptionsSAX(&myHandlerRecord, this) );
-    E( theOutputter -> eventBeginOutput() );
+    E( source.setOptionsSAX(S, &myHandlerRecord, this) );
+    E( source.eventBeginOutput(S) );
     return OK;
 }
 
-eFlag TreeConstructer::parseUsingSAXForAWhile()
+eFlag TreeConstructer::parseUsingSAXForAWhile(Sit S, OutputterObj& source)
 // it's assumed that this is a tree constructer over a dataline
 // (not a SAX one)
 {
-    // removing: 
-    //      assert(!theOutputter);
-    // since parseUsingSaxForAWhile can be called recursively
     assert(theTree && theDataLine);
-    M( theSituation, theOutputter = new OutputterObj(theProcessor) );
     // register the handler with the outputter
-    E( theOutputter -> setOptionsSAX(&myHandlerRecord, this) );
-    E( theProcessor -> pushOutputter(theOutputter) );
-    E( theOutputter -> eventBeginOutput() );
+    E( source.setOptionsSAX(S, &myHandlerRecord, this) );
+    E( source.eventBeginOutput(S) );
     return OK;
 }
 
-eFlag TreeConstructer::parseUsingSAXForAWhileDone()
+eFlag TreeConstructer::parseUsingSAXForAWhileDone(Sit S, OutputterObj& source)
 {
-    assert(theOutputter);
-    E( theOutputter -> eventEndOutput() );
-    E( theProcessor -> popOutputter() ); // deletes the outputter
-    // the following was: 
-    //   theOutputter = NULL;
-    theOutputter = theProcessor -> outputter();
+    E( source.eventEndOutput(S) );
     return OK;
 }
 
-int TreeConstructer::getCurrentLineNumber()
+int TreeConstructer::getCurrentLineNumber() const
 {
     if (theParser)
         return XML_GetCurrentLineNumber(theParser);
     else
         return theLineNumber;
 }
+  
+/* static */
+eFlag TreeConstructer::getDocEncoding(Sit S, const char *buf, Str& theEncoding, TreeConstructer *this_)
+// assumes the parse buffer null-terminated
+{
+    switch (*(const unsigned short*)buf)
+    {
+    case 0xfeff:
+    case 0xfffe:
+        theEncoding = "UTF-16";
+        break;
+    case 0x003c:
+    case 0x3c00:
+        // no byte order mark!
+        if (((const unsigned short*)buf)[1] == 0x003f ||
+            ((const unsigned short*)buf)[1] == 0x3f00)
+            theEncoding = "UTF-16";
+        else
+            this_ -> Warn(S, W_BAD_START);
+        break;
+    case 0x0000:
+        if (((const unsigned short*)buf)[1] == 0x003c ||
+            ((const unsigned short*)buf)[1] == 0x3c00 ) 
+            theEncoding = "ISO-10646-UCS-4";
+        else
+            this_ -> Warn(S, W_BAD_START);
+        break;
+    default:
+        {
+            if (buf[0] == '<' && buf[1] == '?' && buf[2] == 'x' && buf[3] == 'm')
+            {
+                const char *p = buf;
+                p = strpbrk(p + 2,"=?");
+                if (p && *p == '=')
+                {
+                    // found '=' in version, search on
+                    p = strpbrk(p + 1, "=?");
+                    if (p && *p == '=')
+                    {
+                        p++;
+                        skipWhite(p);
+
+                        const char *q = strpbrk(p + 1, "?\'\"");
+                        if (q && *q != '?' && *q == *p)
+                        {
+                            theEncoding.nset(p + 1, q - p - 1);
+                            break;
+                        }
+                    }
+                }
+            }
+            theEncoding = "UTF-8";
+        };
+    }  
+    return OK;
+}
 
 
 /* static */
-eFlag TreeConstructer::feedDocumentToParser(void* constructer)
+eFlag TreeConstructer::feedDocumentToParser(Sit S, void* constructer)
 {
-    Bool quit = FALSE, firstTime = TRUE;
-    int res;
-    int bytes;
-    
     TreeConstructer *this_ =
         (TreeConstructer*) constructer;
-    SituationObj &situation_ = *(this_ -> theSituation);
+    char convBuffer[PARSE_CONV_BUFSIZE];
+    char rawBuffer[PARSE_BUFSIZE + 1];
+    char *outbuf = convBuffer;
+    const char *inbuf = rawBuffer;
+    rawBuffer[PARSE_BUFSIZE] = 0;
 
-    if (situation_.isError())
+    Bool quit = FALSE, firstTime = TRUE;
+    Bool mustConvert = FALSE, haveReadAll = FALSE;
+    int res, bytes;
+    size_t inleft = 0, outleft = 0;
+    EncResult convResult = ENC_OK;
+    CDesc cd = (CDesc) -1;
+    // seems to be useless here
+    // void *recoderUD;
+    // EncHandler *recoder = NZ(S.getProcessor()) -> getEncHandler(&recoderUD);
+    Str theEncoding;
+    
+    if (S.isError())
         return NOT_OK;
     XML_Parser parser = NZ( this_ -> theParser );
 
-    situation_.pushCurrent();
-    situation_.setCurrFile(this_ -> theDataLine -> fullUri);
+    // GP: tree constructer does get killed on error
+    // callers are Processor::parse (stack variable) and ext ent handler (disposes)
 
     // this is to add the 'xml' namespace declarations
     //
@@ -196,39 +251,115 @@ eFlag TreeConstructer::feedDocumentToParser(void* constructer)
 
     while (!quit)
     {
-        char *buf = (char*) XML_GetBuffer(parser, PARSE_BUFSIZE);
-        bytes = this_ -> theDataLine -> get(buf, PARSE_BUFSIZE); 
-        if (situation_.isError())
+//        char *buf = (char*) XML_GetBuffer(parser, PARSE_BUFSIZE + 1);
+        switch (convResult)
         {
+        case ENC_OK:
+            {
+                // just get the next block of data
+                bytes = this_ -> theDataLine -> get(S, rawBuffer, PARSE_BUFSIZE); 
+                haveReadAll = (bytes < PARSE_BUFSIZE);
+                inbuf = rawBuffer;
+                inleft = bytes;
+            }; break;
+        case ENC_EINVAL:
+            {
+                // buffer ended with an incomplete sequence
+                // copy the sequence
+                memmove(rawBuffer, inbuf, inleft);
+                // get the rest from dataline
+                bytes = this_ -> theDataLine -> get(S, rawBuffer + inleft, PARSE_BUFSIZE - inleft); 
+                haveReadAll = (bytes < PARSE_BUFSIZE - (int) inleft);
+                inbuf = rawBuffer;
+                inleft += bytes;
+            }; break;
+        case ENC_E2BIG:
+            {
+                // the converted text was too big to fit in convBuffer
+                // don't get more, just convert from input buffer
+                // not changing haveReadAll, inbuf or inleft
+            }; break;
+        default:
+            assert(!"feedDocumentToParser");
+        }
+
+        // the test for bytes==-1 is superfluous but just in case
+        if (bytes == -1 || S.isError())
+        {
+            // read error, bailing out
             XML_ParserFree(parser);
             return NOT_OK;
         };
-        quit = (Bool) (bytes < PARSE_BUFSIZE);
-        // res = XML_Parse(parser,buf,bytes,quit);
-        res = XML_ParseBuffer(parser,bytes,quit);
-        if (situation_.isError())
+
+        if (firstTime)
+        {
+            // find the document's encoding
+            E( getDocEncoding(S, rawBuffer, theEncoding, this_) );
+            // decide whether to recode or not
+            if (S.recoder().handledByExpat(theEncoding))
+                mustConvert = FALSE;
+            else
+            {
+                mustConvert = TRUE;
+                XML_SetEncoding(parser, "UTF-8");
+                E( S.recoder().openToUTF8(S, theEncoding, cd) );
+            }
+        }
+        
+        if (mustConvert)
+        {
+            outleft = PARSE_CONV_BUFSIZE;
+            outbuf = convBuffer;
+            E( S.recoder().conv(S, cd,
+                inbuf, inleft, outbuf, outleft, 
+                convResult) );
+            switch(convResult)
+            {
+            case ENC_OK:
+                quit = haveReadAll;
+                break;
+            case ENC_EINVAL:
+            case ENC_E2BIG:
+                quit = FALSE;
+                break;
+            case ENC_EILSEQ:
+		        Err1T(this_, S, E1_BAD_CHAR, theEncoding);
+                break;
+            default:
+                assert(!"bad convResult");
+            };
+            bytes = PARSE_CONV_BUFSIZE - outleft;
+            res = XML_Parse(parser,convBuffer,bytes,quit);
+        }
+        else
+        {
+            quit = haveReadAll;
+            res = XML_Parse(parser,rawBuffer,bytes,quit);
+        }
+        if (S.isError())
             return NOT_OK;
         if (!res) 
         {
-            // situation_.setCurrFile(t -> name); - already set
+            // parsing error to be reported
+            // situation_.setCurrFile(t -> name);   is unnecessary as already set
+
             // hack to avoid an apparent bug in expat causing crashes when an UTF-8 text
             // happens to start with a byte order mark by mistake
-            if (!(firstTime && buf[0] == (char) 0xEF && buf[1] == (char) 0xBB && buf[2] == (char) 0xBF))
-                situation_.setCurrLine(XML_GetCurrentLineNumber(parser));
+            if (!(firstTime && rawBuffer[0] == (char) 0xEF && 
+                rawBuffer[1] == (char) 0xBB && rawBuffer[2] == (char) 0xBF))
+                S.setCurrLine(XML_GetCurrentLineNumber(parser));
             int code = XML_GetErrorCode(parser); 
             Str eCodeStr, eNameStr;
             eCodeStr = code;
             eNameStr = (char*) XML_ErrorString(code);
-            Err2(this_ -> theSituation, E_XML, eCodeStr, eNameStr);
-            XML_ParserFree(parser);
+            // XML_ParserFree(parser); -- done later
+            Err2T(this_, S, E_XML, eCodeStr, eNameStr);
         }
         firstTime = FALSE;
     }
     // remove the 'xml' namespace declarations
     //
     tcEndNamespace(constructer, "xml");
-
-    situation_.popCurrent();
     return OK;
 }
 
@@ -241,13 +372,6 @@ eFlag TreeConstructer::feedDocumentToParser(void* constructer)
 void TreeConstructer::tcStartDocument(
     void* constructer)
 {
-/*
-    TreeConstructer *this_ =
-        (TreeConstructer*) constructer;
-    if (this_ -> theSituation -> isError())
-        return;
-    Tree *t = this_ -> theTree;
-*/
 };
 
 
@@ -268,56 +392,55 @@ void TreeConstructer::tcStartElement(
     XSL_OP opCode;
     XSL_ATT attCode;
     BOOL itsXSL = FALSE;
-    Vertex *v;
-    Attribute *a;
-    QName q(this_ -> theProcessor);
+    Vertex *v;  // GP: OK (goes to tree)
+    Attribute *a;   // GP: OK (goes to tree)
+    QName q;
     int elemLine;
 
-    if (this_ -> theSituation -> isError())
+    if (this_ -> S.isError())
         return;
-    this_ -> theSituation -> setCurrLine(
+    this_ -> S.setCurrLine(
         elemLine = this_ -> getCurrentLineNumber());
-    if (setQNameFromExpat(this_, q, elName))
+    if (setQNameFromExpat(this_ -> S, this_, q, elName))
         return;
-    if ((t -> XSLTree) && (q.uri == this_ -> theProcessor -> stdPhrase(PHRASE_XSL_NAMESPACE)))
+
+	// DETACH: optimize this comparison ###
+    if ((t -> XSLTree) && (t -> expand(q.getUri()) == theXSLTNamespace)) //t -> stdPhrase(PHRASE_XSL_NAMESPACE)))
     {
         itsXSL = TRUE;
-        opCode = (XSL_OP) lookup((char*) q.getLocal(),xslOpNames);
+        opCode = (XSL_OP) lookup((char*)(t -> expand(q.getLocal())),xslOpNames);
         if (opCode == XSL_NONE)
         {
-            this_ -> theSituation -> error(ET_BAD_XSL, 
-					   "" /**theEmptyString*/, 
-					   "" /**theEmptyString*/);
+            this_ -> S.message(MT_ERROR, ET_BAD_XSL, (char*)NULL, (char*)NULL);
             return;
         };
-        v = new(this_ -> theProcessor -> getArena()) XSLElement(q, t, opCode, this_ -> theProcessor);
+        v = new(&(t -> getArena())) XSLElement(*t, q, opCode);
     }
     else
-        v = new(this_ -> theProcessor -> getArena()) Element(q, t, this_ -> theProcessor);
+        v = new(&(t -> getArena())) Element(*t, q);
     v -> lineno = elemLine;
-    t -> appendVertex(v);
-    t -> pendingNS.giveCurrent(toE(v) -> namespaces, t);
-    toE(v) -> name.findPrefix(toE(v) -> namespaces);
+    t -> appendVertex(this_ -> S, v);
+    t -> pendingNS.giveCurrent(this_ -> S, toE(v) -> namespaces, t);
+    toE(v) -> namespaces.findPrefix(toE(v) -> name);
     
     while(*p)
     {
-        if (setQNameFromExpat(this_, q, (char *)p[0]))
+        if (setQNameFromExpat(this_ -> S, this_, q, (char *)p[0]))
             return;
-        q.findPrefix(toE(v) -> namespaces);
+        toE(v) -> namespaces.findPrefix(q);
         attCode = (itsXSL ? 
-            (XSL_ATT) lookup((char*)q.getLocal(),xslAttNames) : XSLA_NONE);
-        a = new(this_ -> theProcessor -> getArena()) 
-            Attribute(q,p[1],attCode,this_ -> theProcessor);
+            (XSL_ATT) lookup(t -> expand(q.getLocal()),xslAttNames) : XSLA_NONE);
+        a = new(&(t -> getArena())) Attribute(*t, q, p[1],attCode);
         a -> lineno = this_ -> getCurrentLineNumber();
-        t -> appendVertex(a);
+        t -> appendVertex(this_ -> S, a);
         p += 2;
     };
     
     if (itsXSL)
     {
-        toX(v) -> checkAtts(); 
+        if (toX(v) -> checkAtts(this_ -> S)) return; 
         // also check if a toplevel element does not have a non-stylesheet parent
-        toX(v) -> checkToplevel(); 
+        if (toX(v) -> checkToplevel(this_ -> S)) return; 
     }
     else
     {
@@ -326,8 +449,8 @@ void TreeConstructer::tcStartElement(
             int k, 
                 kLimit = toE(v) -> atts.number();
             for (k = 0; k < kLimit; k++)
-                toA(toE(v) -> atts[k]) -> buildExpr(
-                    TRUE, EX_NONE);
+                if (toA(toE(v) -> atts[k]) -> buildExpr(this_ -> S,
+                    TRUE, EX_NONE)) return;
             // this calls situation.error() on error
         }
     }
@@ -345,12 +468,11 @@ void TreeConstructer::tcEndElement(
 {
     TreeConstructer *this_ =
         (TreeConstructer*) constructer;
-    if (this_ -> theSituation -> isError())
+    if (this_ -> S.isError())
         return;
     Tree *t = this_ -> theTree;
 
     Vertex *v = NZ( t -> stackTop );
-
     t -> flushPendingText();
     
     if (t -> XSLTree)
@@ -361,13 +483,11 @@ void TreeConstructer::tcEndElement(
     if(isXSLElement(v))
     {
         // situation.error() is called in the following
-        if (toX(v) -> checkChildren())
+        if (toX(v) -> checkChildren(this_ -> S))
             return;
     }
-    if (this_ -> theProcessor -> processVertexAfterParse(v, t, this_))
-        return;
-    
-    // t -> popVertex();    this is done in processVertexAfterParse
+
+	t -> processVertexAfterParse(this_ -> S, v, this_);        
 }
 
 
@@ -382,32 +502,32 @@ void TreeConstructer::tcStartNamespace(
 {
     TreeConstructer *this_ =
         (TreeConstructer*) constructer;
-    if (this_ -> theSituation -> isError())
+    if (this_ -> S.isError())
         return;
     Tree *t = this_ -> theTree;
 
-    Vertex *newv;
+    Vertex *newv;   // GP: OK (goes to tree)
 
     Phrase prefixPh, uriPh;
     if (prefix)
-        this_ -> theProcessor -> dict().insert(prefix, prefixPh);
+        t -> dict().insert(prefix, prefixPh);
     else
         prefixPh = UNDEF_PHRASE;
-    this_ -> theProcessor -> dict().insert(uri, uriPh);
+    t -> dict().insert(uri, uriPh);
 
-    t -> pendingNS.appendAndSetOrdinal(
-        newv = new(this_ -> theProcessor -> getArena()) NmSpace(
-            prefixPh, uriPh, this_ -> theProcessor));
+    // t -> pendingNS.appendAndSetOrdinal(
+    t -> pendingNS.append(
+        newv = new(&(t -> getArena())) NmSpace(*t, prefixPh, uriPh));
     newv -> lineno = this_ -> getCurrentLineNumber();
     
     // warn on obsolete namespace
     if (uri && !strcmp(oldXSLTNamespace, uri)) /* _PH_ */
-        Warn1(this_ -> theSituation, W1_OLD_NS_USED, (char*)uri)
+        this_ -> Warn1(this_ -> S, W1_OLD_NS_USED, (char*)uri)
     else
     {
         if (prefix && !strcmp(prefix, "xsl") && 
 	      uri && strcmp(theXSLTNamespace, uri)) /* _PH_ */
-	  Warn1(this_ -> theSituation, W1_XSL_NOT_XSL, (char*) uri);
+	  this_ -> Warn1(this_ -> S, W1_XSL_NOT_XSL, (char*) uri);
     }
 };
 
@@ -424,7 +544,7 @@ void TreeConstructer::tcEndNamespace(
 {
     TreeConstructer *this_ =
         (TreeConstructer*) constructer;
-    if (this_ -> theSituation -> isError())
+    if (this_ -> S.isError())
         return;
     Tree *t = this_ -> theTree;
 
@@ -432,7 +552,7 @@ void TreeConstructer::tcEndNamespace(
     // hope this works
     Phrase prefixPh;
     if (prefix)
-        prefixPh = this_ -> theProcessor -> dict().lookup(prefix);
+        prefixPh = t -> dict().lookup(prefix);
     else
         prefixPh = UNDEF_PHRASE;
     assert(toNS(t -> pendingNS.last()) -> prefix == prefixPh);
@@ -452,17 +572,17 @@ void TreeConstructer::tcComment(
 {
     TreeConstructer *this_ =
         (TreeConstructer*) constructer;
-    if (this_ -> theSituation -> isError())
+    if (this_ -> S.isError())
         return;
     Tree *t = this_ -> theTree;
 
     if (t -> XSLTree)
         return;
 
-    Comment *newNode;
-    newNode = new(this_ -> theProcessor -> getArena()) Comment(contents, this_ -> theProcessor);
+    Comment *newNode;   // GP: OK
+    newNode = new(&(t -> getArena())) Comment(*t, contents);
     newNode -> lineno = this_ -> getCurrentLineNumber();
-    t -> appendVertex(newNode);
+    t -> appendVertex(this_ -> S, newNode);
 };
 
 
@@ -477,21 +597,21 @@ void TreeConstructer::tcPI(
 {
     TreeConstructer *this_ =
         (TreeConstructer*) constructer;
-    if (this_ -> theSituation -> isError())
+    if (this_ -> S.isError())
         return;
     Tree *t = this_ -> theTree;
 
     if (t -> XSLTree)
         return;
 
-    ProcInstr *newNode;
+    ProcInstr *newNode; // GP: OK
     Phrase targetPh;
-    this_ -> theProcessor -> dict().insert(target, targetPh);
+    t -> dict().insert(target, targetPh);
 
-    newNode = new(this_ -> theProcessor -> getArena())
-        ProcInstr(targetPh, contents, this_ -> theProcessor);
+    newNode = new(&(t -> getArena()))
+        ProcInstr(*t, targetPh, contents);
     newNode -> lineno = this_ -> getCurrentLineNumber();
-    t -> appendVertex(newNode);
+    t -> appendVertex(this_ -> S, newNode);
 };
 
 
@@ -506,12 +626,12 @@ void TreeConstructer::tcCharacters(
 {
     TreeConstructer *this_ =
         (TreeConstructer*) constructer;
-    if (this_ -> theSituation -> isError())
+    if (this_ -> S.isError())
         return;
     Tree *t = this_ -> theTree;
 
-    Vertex *newVertex;
-    if (!!(newVertex = t -> appendText((char *) contents, length)))
+    Vertex *newVertex;  // GP: OK
+    if (!!(newVertex = t -> appendText(this_ -> S, (char *) contents, length)))
         newVertex -> lineno = this_ -> getCurrentLineNumber();
 };
 
@@ -526,13 +646,6 @@ void TreeConstructer::tcCharacters(
 void TreeConstructer::tcEndDocument(
     void* constructer)
 {
-/*
-    TreeConstructer *this_ =
-        (TreeConstructer*) constructer;
-    if (this_ -> theSituation -> isError())
-        return;
-    Tree *t = this_ -> theTree;
-*/
 };
 
 
@@ -543,7 +656,9 @@ void TreeConstructer::tcEndDocument(
 //
 
 
+// the unknown encoding handler is no more used:
 /* static */
+/*
 int TreeConstructer::tcUnknownEncoding(
     void *encodingHandlerData, const char *name, XML_Encoding *info)
 {
@@ -566,6 +681,7 @@ int TreeConstructer::tcUnknownEncoding(
     info -> release = NULL;
     return 1;
 };
+*/
 
 //
 //  tcExternalEntity
@@ -579,56 +695,59 @@ int TreeConstructer::tcExternalEntityRef(
 {
     TreeConstructer *this_ =
         (TreeConstructer*) XML_GetUserData(parser);
-    if (this_ -> theSituation -> isError())
+    if (this_ -> S.isError())
         return 0;
     Tree *t = this_ -> theTree;
 
-    SituationObj* situation = this_ -> theSituation;
-
-    Log1(situation, L1_READING_EXT_ENTITY, systemId);    
+    this_ -> Log1(this_ -> S, L1_READING_EXT_ENTITY, systemId);    
     XML_Parser newParser =
         XML_ExternalEntityParserCreate(parser, context, /* encoding= */ NULL);
+    if (!newParser)
+        return 0;
+
     DataLine *newDL;
     Str absolute;
     E( makeAbsoluteURI(systemId, base, absolute) );
-    E( this_ -> theProcessor -> addLineNoTree(newDL, absolute, t -> XSLTree) );
-    if (!newParser || !newDL)
+    E( NZ(this_ -> S.getProcessor()) -> addLineNoTree(this_ -> S, newDL, absolute, t -> XSLTree) );
+    if (!newDL)
+    {
+        XML_ParserFree(newParser);
         return 0;
+    }
 
     TreeConstructer *newTC;
-    M( situation, newTC = new TreeConstructer(this_ -> theProcessor));
-    if (newTC -> parseDataLineUsingGivenExpat(t, newDL, newParser))
-        return 0;
-    
+    MT( this_, this_ -> S, newTC = new TreeConstructer(this_ -> S));
+    eFlag code = newTC -> parseDataLineUsingGivenExpat(this_ -> S, t, newDL, newParser);
     XML_ParserFree(newParser);
     delete newTC;
-    return 1;
+    return code ? 0 : 1;
 }
 
 /* static */
-eFlag TreeConstructer::setQNameFromExpat(TreeConstructer* this_, QName& qname_, const char* text)
+eFlag TreeConstructer::setQNameFromExpat(Sit S, TreeConstructer* this_, QName& qname_, const char* text)
 {
+    Tree *t = this_ -> theTree;
     char *p = (char*) text,
         *q = strchr(p, THE_NAMESPACE_SEPARATOR);
     if (q)
     {
         *q = 0;
-        qname_.setUri(p);
+        qname_.setUri(t -> unexpand(p));
         *q = NS_SEP;
-        qname_.setLocal(q+1);
+        qname_.setLocal(t -> unexpand(q+1));
         if (strchr(q+1,':'))
         {
             DStr msg = "{";
-            msg += qname_.getUri();
+            msg += t -> expand(qname_.getUri());
             msg += "}:";
-            msg += qname_.getLocal();
-            Err1(this_ -> theProcessor -> situation, E1_EXTRA_COLON, (char *)msg);
+            msg += t -> expand(qname_.getLocal());
+            Err1T(this_, S, E1_EXTRA_COLON, (char *)msg);
         }
     }
     else
     {
         qname_.uri = UNDEF_PHRASE;
-        qname_.setLocal(p);
+        qname_.setLocal(t -> unexpand(p));
         char *isColon;
         qname_.prefix = UNDEF_PHRASE;
         if (!!(isColon = strchr(p,':')))
@@ -638,19 +757,25 @@ eFlag TreeConstructer::setQNameFromExpat(TreeConstructer* this_, QName& qname_,
             // fixing what appears a bug in expat - sometimes the xml:lang attr arrives unexpanded
             // apparently only in XSL and when it's not at the top level
             if (!strEqNoCase(p,"xml"))
-                Err1(this_ -> theProcessor -> situation, ET_BAD_PREFIX,p)
+                Err1T(this_, S, ET_BAD_PREFIX,p)
             else
             {
-                qname_.setLocal(isColon + 1);
-                qname_.uri = this_ -> theProcessor -> stdPhrase(PHRASE_XML_NAMESPACE);
-                E( this_ -> theProcessor -> dict().insert("xml",qname_.prefix) );                
+                qname_.setLocal(t -> unexpand(isColon + 1));
+                qname_.setUri(t -> stdPhrase(PHRASE_XML_NAMESPACE));
+		        qname_.setPrefix(t -> unexpand((char*)"xml"));
+                // E( t -> dict().insert(S, "xml",qname_.getPrefix) );                
             }
         }
     };
     return OK;
 }
 
-OutputterObj *TreeConstructer::getOutputter()
+void TreeConstructer::report(Sit S, MsgType type, MsgCode code, 
+    const Str& arg1, const Str& arg2) const 
 {
-    return theOutputter;
-}
+    if (theDataLine)
+        S.setCurrFile(theDataLine -> fullUri);
+	S.setCurrLine(getCurrentLineNumber());
+    S.message(type, code, arg1, arg2);
+};
+
