@@ -36,6 +36,8 @@
 #ifndef ParserHIncl
 #define ParserHIncl
 
+// GP: clean
+
 #include "base.h"
 #include "output.h"
 
@@ -51,22 +53,20 @@
 
 class Tree;
 class DataLine;
-class Processor;
 
 // TreeConstructer
 
 class TreeConstructer
 {
 public:
-    TreeConstructer(Processor *theProcessor_);
+    TreeConstructer(Sit S_);
     ~TreeConstructer();
-    eFlag parseDataLineUsingExpat(
+    eFlag parseDataLineUsingExpat(Sit S,
         Tree *t, DataLine *d);
-    eFlag parseUsingSAX(
-        Tree *t);
+    eFlag parseUsingSAX(Sit S, Tree *t, OutputterObj& source);
 
-    eFlag parseUsingSAXForAWhile();
-    eFlag parseUsingSAXForAWhileDone();
+    eFlag parseUsingSAXForAWhile(Sit S, OutputterObj& source);
+    eFlag parseUsingSAXForAWhileDone(Sit S, OutputterObj& source);
 
     static void tcStartDocument(
         void* userData);
@@ -86,26 +86,27 @@ public:
         void* constructer, const char* contents, int length);
     static void tcEndDocument(
         void* constructer);
-    static int tcUnknownEncoding(
-        void *encodingHandlerData, const char *name, XML_Encoding *info);
+    // the unknown encoding handler is no more used:
+    // static int tcUnknownEncoding(
+    //     void *encodingHandlerData, const char *name, XML_Encoding *info);
     static int tcExternalEntityRef(
         XML_Parser parser, const char* context, const char* base,
         const char* systemId, const char* publicId);
-    OutputterObj *getOutputter();
 
 private:
-    eFlag parseDataLineUsingGivenExpat(Tree *t, DataLine *d, XML_Parser theParser_);
-    static eFlag setQNameFromExpat(TreeConstructer* this_, QName& qname_, const char* text);
-    static eFlag feedDocumentToParser(void* constructer);
-    int getCurrentLineNumber();
-    Processor *theProcessor; 
+    Situation& S;
+    eFlag parseDataLineUsingGivenExpat(Sit S, Tree *t, DataLine *d, XML_Parser theParser_);
+    static eFlag setQNameFromExpat(Sit S, TreeConstructer* this_, QName& qname_, const char* text);
+    static eFlag feedDocumentToParser(Sit S, void* constructer);
+    static eFlag getDocEncoding(Sit S, const char *buf, Str& theEncoding, TreeConstructer *this_);
+    int getCurrentLineNumber() const;
     XML_Parser theParser;
-    SituationObj *theSituation;
-    OutputterObj *theOutputter;
     Tree *theTree;
     DataLine *theDataLine;
     int theLineNumber;
     static SAXHandler myHandlerRecord; // initialized in parser.h
+    void report(Sit S, MsgType type, MsgCode code, 
+        const Str& arg1, const Str& arg2) const;
 };
 
 #endif
