@@ -38,19 +38,9 @@
 #include "vars.h"
 // #include "xmlparse.h" - moved to parser.h
 #include "parser.h"
+#include "guard.h"
 
-/*
-  include direct.h for getcwd()
-*/
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#elif defined(WIN32)
-#include <direct.h>
-#elif defined(__BORLANDC__)
-#include <dir.h>
-#else
-#error "Can't find appropriate include (unistd.h & Co.)"
-#endif
+// GP: clean
 
 /*****************************************************************
 R u l e I t e m   methods
@@ -78,9 +68,9 @@ RuleItem::~RuleItem()
 R u l e S L i s t   methods
 *****************************************************************/
 
-RuleSList::RuleSList(Processor* _proc)
+RuleSList::RuleSList()
 :
-SList<RuleItem*>(LIST_SIZE_LARGE), proc(_proc)
+SList<RuleItem*>(LIST_SIZE_LARGE)
 {};
 
 RuleSList::~RuleSList()
@@ -94,11 +84,11 @@ int RuleSList::compare(int first, int second)
 };
 
 
-XSLElement *RuleSList::findByName(QName &what)
+XSLElement *RuleSList::findByName(const Tree& t, const QName &what) const
 {
     int theNumber = number();
     for (int i=0; i < theNumber; i++)
-        if (proc -> cmpQNames((*this)[i] -> name, what))
+        if (t.cmpQNames((*this)[i] -> name, what))
             return (*this)[i] -> rule;
     return NULL;
 }
@@ -109,13 +99,19 @@ XSLElement *RuleSList::findByName(QName &what)
 
 *****************************************************************/
 
+DataLineItem::DataLineItem(Sit S_)
+: situation(S_)
+{
+}
+
 DataLineItem::~DataLineItem()
 {
     // the following close should be checked for error!!
     if (_dataline && _dataline -> mode != DLMODE_CLOSED)
-        _dataline -> close();
+        _dataline -> close(situation);
     cdelete(_dataline);
-    cdelete(_tree);
+    if (!_preparsedTree)
+        cdelete(_tree);
 }
 
 /*****************************************************************
@@ -133,7 +129,8 @@ int DataLinesList::findNum(Str &absoluteURI, Bool _isXSL,
         DataLineItem* item = (*this)[i];
         if ((item->_dataline -> fullUri == absoluteURI) &&
             (item->_isXSL == _isXSL) && 
-            (item-> _dataline -> mode == _mode))
+            (item-> _dataline -> mode == _mode || 
+	         item-> _dataline -> mode == DLMODE_CLOSED))
         return i;
     }
     return -1;
@@ -149,12 +146,14 @@ Tree *DataLinesList::getTree(Str &absoluteURI, Bool _isXSL,
         return NULL;
 }
 
-eFlag DataLinesList::addLine(DataLine *d, Tree *t, Bool isXSL)
+eFlag DataLinesList::addLine(Sit S, DataLine *d, Tree *t, Bool isXSL,
+    Bool preparsedTree /* = FALSE */)
 {
-    DataLineItem *item = new DataLineItem;
+    DataLineItem *item = new DataLineItem(S);
     item -> _dataline = d;
     item -> _tree = t;
     item -> _isXSL = isXSL;
+    item -> _preparsedTree = preparsedTree;
     append(item);
     return OK;
 }
@@ -173,9 +172,7 @@ eFlag DataLinesList::addLine(DataLine *d, Tree *t, Bool isXSL)
 
 Processor::Processor() 
 : 
-situation_( this ), rules ( this ), theEmptyQName( this ), 
-theArena(this, PROC_ARENA_SIZE),
-dictionary( this, &theArena, PROC_DICTIONARY_LOGSIZE )
+theArena(PROC_ARENA_SIZE)
 
 #ifdef WIN32
 #pragma warning( default : 4355 )
@@ -183,28 +180,25 @@ dictionary( this, &theArena, PROC_DICTIONARY_LOGSIZE )
 
 {
 //    pushNewHandler();
-
     theSchemeHandler = NULL;
     theMessageHandler = NULL;
     theSAXHandler = NULL;
     theMiscHandler = NULL;
+    theEncHandler = NULL;
     theSchemeUserData = theMessageUserData = 
-        theSAXUserData = theMiscUserData = NULL;
-    
+        theSAXUserData = theMiscUserData = theEncUserData = NULL;    
     instanceData = NULL;
-    situation = &situation_;
+    instanceSituation = NULL;
+    addedFlag = FALSE;
+    vars = NULL;
+    input = styleSheet = NULL;
 };
 
 void Processor::prepareForRun()
 {
-    vars = new VarsList(this);
+    // do this in open():
+    // vars = new VarsList();
     input = styleSheet = NULL;
-    dictionary.initialize();
-    dictionary.insert("", stdPhrases[PHRASE_EMPTY]);
-    dictionary.insert("xsl", stdPhrases[PHRASE_XSL]);
-    dictionary.insert(theXSLTNamespace, stdPhrases[PHRASE_XSL_NAMESPACE]);
-    dictionary.insert(theXMLNamespace, stdPhrases[PHRASE_XML_NAMESPACE]);
-    dictionary.insert("*", stdPhrases[PHRASE_STAR]);
 }
 
 void Processor::freeNonArgDatalines()
@@ -213,18 +207,24 @@ void Processor::freeNonArgDatalines()
     while(i < datalines.number())
     {
         if (datalines[i] -> _dataline -> scheme != URI_ARG)
+	    {
             datalines.freerm(i, FALSE);
+	    }
         else
         {
             // removing the tree here because the arena gets disposed
             // in cleanupAfterRun(). So only the arg buffers remain.
-            cdelete(datalines[i] -> _tree);
-            i++;
-        }
+	        // only remove the tree if not preparsed
+		    if (!datalines[i] -> _preparsedTree)
+                cdelete(datalines[i] -> _tree);
+	        // don't move the following line out of the else!
+            i++;        	
+        };
     }
+    addedFlag = FALSE;
 }
 
-void Processor::cleanupAfterRun()
+void Processor::cleanupAfterRun(Sit S)
 {
 /*
 These deletes are now subsumed in "datalines.freeall()":
@@ -234,7 +234,7 @@ These deletes are now subsumed in "datalines.freeall()":
 */
     cdelete(vars);
     freeNonArgDatalines();
-    if (!situation->isError())
+    if (!S.isError())
     {
         assert(modes.isEmpty());
         assert(outputters_.isEmpty());
@@ -244,10 +244,10 @@ These deletes are now subsumed in "datalines.freeall()":
         modes.freeall(FALSE);
         outputters_.freeall(FALSE);
     };
-    rules.freeall(FALSE);
-    situation->clear();
+    S.clear();
     // hope this works:
-    dictionary.destroy();
+    //DETACH: move to Tree
+    // dictionary.destroy();
     theArena.dispose();
 }
 
@@ -255,22 +255,11 @@ Processor::~Processor()
 {
 //    popNewHandler();
     baseURIMappings.freeall(FALSE);
+    // FIXME: MUST clear theRecoder but got no situation
+//    theRecoder.clear();
 };
 
-void my_getcwd(DStr &dir)
-{
-char buf[256];
-#if defined(WIN32)
-    _getcwd(buf, 256);
-#else //if defined(__linux__) || defined(__BORLANDC__) || defined(__unix)
-    getcwd(buf,256);
-#endif
-    dir = buf;
-    if (!(dir == (const char*) "/"))
-        dir += '/';
-}
-
-eFlag Processor::open(char *sheetURI, char *inputURI, char *resultURI)
+eFlag Processor::open(Sit S, char *sheetURI, char *inputURI)
 {
     Str temp;
     DStr theBase;
@@ -278,28 +267,38 @@ eFlag Processor::open(char *sheetURI, char *inputURI, char *resultURI)
     my_getcwd(theBase);
     theBase = findBaseURI(Str("file://") + theBase);
 
-    E( readTreeFromURI(styleSheet, temp = sheetURI, theBase, TRUE) );
-    E( readTreeFromURI(input, temp = inputURI, theBase, FALSE) );
+    E( readTreeFromURI(S, styleSheet, temp = sheetURI, theBase, TRUE) );
+    E( readTreeFromURI(S, input, temp = inputURI, theBase, FALSE) );
+    vars = new VarsList (*styleSheet);    
+    return OK;
+}
 
-    E( pushOutputterForURI(temp = resultURI, theBase) );
-    E( outputter() -> eventBeginOutput() );
+eFlag Processor::run(Sit S, char* resultURI)
+{
+    Str temp;
+    DStr theBase;
+    my_getcwd(theBase);
+    theBase = findBaseURI(Str("file://") + theBase);
 
-    Log1(situation, L1_EXECUTING, styleSheet -> name);
+    Log1(S, L1_EXECUTING, styleSheet -> name);
     double time_was = getMillisecs();
-    situation->setCurrFile(styleSheet -> name);
 
-    Context *c = new Context(this);
-    c -> set(&(input -> root));
+    E( pushOutputterForURI(S, temp = resultURI, theBase) );
+    E( outputter() -> eventBeginOutput(S) );
+
+    GP( Context ) c = new Context;
+    (*c).set(&(input -> getRoot()));
 
     /*
         in vars, global prebindings go to call level 0,
         global bindings to call level 1
     */
     vars -> startCall();
-    E( styleSheet -> root.execute(c) );
+    E( styleSheet -> getRoot().execute(S, c) );
     vars -> endCall();
+    c.del();
 
-    E( outputter() -> eventEndOutput() );
+    E( outputter() -> eventEndOutput(S) );
         
     // report info about the output document to the MiscHandler
     void *miscUserData;
@@ -312,13 +311,13 @@ eFlag Processor::open(char *sheetURI, char *inputURI, char *resultURI)
             outDef -> getValueStr(XSLA_MEDIA_TYPE), 
             outDef -> getValueStr(XSLA_ENCODING));
 
-    E( popOutputter() );
-    delete c;
-    Log1(situation, L1_EXECUTION_DONE, getMillisecsDiff(time_was));
+    E( popOutputter(S) );
+    Log1(S, L1_EXECUTION_DONE, getMillisecsDiff(time_was));
     return OK;
 }
 
 
+
 /*================================================================
 getVarBinding 
     finds the current binding for the variable with the given name.
@@ -335,35 +334,37 @@ cmpQNames
     compares two given QNames based on the namespace URIs,
     allowing wildcard * as local name of first QName.
 ================================================================*/
-
+/*
 Bool Processor::cmpQNames(const QName& first, const QName &second) const
 {
-    if (first.local == stdPhrase(PHRASE_STAR))
-        return (Bool)(first.prefix == UNDEF_PHRASE || 
-            (first.uri == second.uri));
+    if (first.getLocal() == stdPhrase(PHRASE_STAR))
+        return (Bool)(first.getPrefix() == UNDEF_PHRASE || 
+            (first.getUri() == second.getUri()));
     else
         return (Bool) (first == second);
 }
+*/
 
 /*================================================================
 execute
+both flavours DEALLOCATE the context and set the pointer to NULL
 ================================================================*/
 
-eFlag Processor::execute(Vertex *IP, Context *c)
+eFlag Processor::execute(Sit S, Vertex *IP, Context *&c)
 {
     while (!c -> isFinished())
     {
         if (IP)
-            E( IP -> execute(c) )
+            E( IP -> execute(S, c) )
         else
-            E( execApply(c) );
+            E( execApply(S, c) );
         c -> shift();
     };
-    delete c;
+    cdelete(c);
     return OK;
 }
 
-eFlag Processor::execute(VertexList &IPlist, Context *c)
+eFlag Processor::execute(Sit S, VertexList &IPlist, Context *&c)
 {
     // we may need to remove some variable bindings on each pass
     // through a for loop
@@ -376,23 +377,23 @@ eFlag Processor::execute(VertexList &IPlist, Context *c)
     
     while (c -> current())
     {
-        E( IPlist.execute(c) );
+        E( IPlist.execute(S, c) );
         c -> shift();
         if (theForParent)
-            theForParent -> removeBindings();
+            theForParent -> removeBindings(S);
     };
-    delete c;
+    cdelete(c);
     return OK;
 }
 
-eFlag Processor::execApply(Context *c)
+eFlag Processor::execApply(Sit S, Context *c)
 {
     XSLElement *rule;
-    E( bestRule(rule, c) );
+    E( bestRule(S, rule, c) );
     if (!rule)
-        E( builtinRule(c) )
+        E( builtinRule(S, c) )
     else
-        E( rule -> execute(c) );
+        E( rule -> execute(S, c) );
     return OK;
 }
 
@@ -404,11 +405,11 @@ bestRule
     If no rule is found, returns NULL.
 ================================================================*/
 
-eFlag Processor::bestRule(XSLElement *& ret, Context *c)
+eFlag Processor::bestRule(Sit S, XSLElement *& ret, Context *c)
 {
     int i;
     int bestRule = -1,
-        rulesNumber = rules.number();
+        rulesNumber = styleSheet -> rules().number();
     double bestPrio=0;
     Expression *pattern = NULL;
     QName 
@@ -419,16 +420,16 @@ eFlag Processor::bestRule(XSLElement *& ret, Context *c)
     {
         if (
             (bestRule != -1) &&
-            (fcomp(rules[i] -> priority, bestPrio) == -1)  //current < best
+            (fcomp(styleSheet -> rules()[i] -> priority, bestPrio) == -1)  //current < best
             )
             break;
-        Attribute *a = (therule = rules[i] -> rule) -> atts.find(XSLA_MATCH);
+        Attribute *a = (therule = styleSheet -> rules()[i] -> rule) -> atts.find(XSLA_MATCH);
         if (a) 
             pattern = a -> expr;
         else 
             continue;
 
-        thisMode = rules[i] -> mode;
+        thisMode = styleSheet -> rules()[i] -> mode;
         if ((!thisMode) ^ (!currMode))
             continue;
         if (thisMode && !((*thisMode) == (*currMode)))
@@ -437,20 +438,20 @@ eFlag Processor::bestRule(XSLElement *& ret, Context *c)
         if (pattern)
         {
             Bool result;
-            E( pattern -> matchesPattern(c, result) );
+            E( pattern -> matchesPattern(S, c, result) );
             if (result)
             {
                 bestRule = i;
-                bestPrio = rules[i] -> priority;
+                bestPrio = styleSheet -> rules()[i] -> priority;
             }
         };
     };
     if (bestRule == -1) ret = NULL;
-    else ret = rules[bestRule] -> rule;
+    else ret = styleSheet -> rules()[bestRule] -> rule;
     return OK;
 };
 
-eFlag Processor::builtinRule(Context *c)
+eFlag Processor::builtinRule(Sit S, Context *c)
 {
     Vertex *v = c -> current();
     
@@ -460,22 +461,23 @@ eFlag Processor::builtinRule(Context *c)
     case VT_ELEMENT:
         //apply-templates
         {
-            Expression *e = new Expression(NULL,this, EXF_LOCPATH);
-            e -> setLS(
-                AXIS_CHILD, 
-                EXNODE_NODE);
-            Context *newc;
-            E( e -> createContext(newc = c) );
-            E( execute(NULL, newc) );
-            delete e; ///
+            GP( Expression ) e = new Expression(*toE(v), EXF_LOCPATH);
+            (*e).setLS(AXIS_CHILD, EXNODE_NODE);
+            GP( Context ) newc;
+            newc.assign(c);
+            E( (*e).createContext(S, newc) );
+            newc.unkeep();
+            E( execute(S, NULL, newc) );
+            newc.keep(); // GP: it's NULL by this time anyway
+            e.del();
         }; break;
     case VT_TEXT:
     case VT_ATTRIBUTE:
         //copy contents to output
         {
             DStr temp;
-            E( v -> value(temp, c) );
-            E( this -> outputter() -> eventData(temp) );
+            E( v -> value(S, temp, c) );
+            E( outputter() -> eventData(S, temp) );
         }; break;
     case VT_PI:
     case VT_COMMENT:
@@ -484,100 +486,6 @@ eFlag Processor::builtinRule(Context *c)
     return OK;
 };
 
-eFlag Processor::insertRule(XSLElement *tmpl)
-{
-    double prio;
-    Attribute *a = tmpl -> atts.find(XSLA_PRIORITY);
-    if (!a)
-        prio = defaultPriority(tmpl);
-    else
-    {
-        if (a -> cont.toDouble(prio))
-            Err(situation, ET_BAD_NUMBER);
-    };
-    QName q(this), *mode = NULL;
-    if (!!(a = tmpl -> atts.find(XSLA_NAME)))
-        E( q.setLogical(a -> cont, &(tmpl -> namespaces), FALSE) );
-    if (!q.getLocal().isEmpty() && rules.findByName(q))
-        Err1(situation, ET_DUPLICATE_RULE_NAME,(char*)(q.getname()));
-    if (!!(a = tmpl -> atts.find(XSLA_MODE)))
-        E( (mode=new QName(this)) -> setLogical(a -> cont, 
-            &(tmpl -> namespaces), FALSE) );
-    
-    rules.insert(new RuleItem(tmpl,prio,q,mode));
-    return OK;
-}
-
-double Processor::defaultPriorityLP(Expression *lpath)
-{
-    assert(lpath && lpath -> functor == EXF_LOCPATH);
-    assert(lpath -> args.number());
-    if ((lpath -> args.number() > 1) || lpath -> args[0] -> step -> preds.number())
-        return .5;
-    else
-    {
-        switch (lpath -> args[0] -> step -> ntype)
-        {
-        case EXNODE_COMMENT:
-        case EXNODE_TEXT:
-        case EXNODE_PI:  //// pi('literal') should have priority 0.5
-        case EXNODE_NODE:
-            return -0.5;
-            break;
-        case EXNODE_NONE:
-            {
-                QName &qn = lpath -> args[0] -> step -> ntest;
-                if (qn.local != stdPhrase(PHRASE_STAR))
-                    return 0.0;
-                else
-                {   
-                    if (qn.prefix == UNDEF_PHRASE)
-                        return -0.5;
-                    else
-                        return -0.25;
-                };
-            }; break;
-        default:
-            return 0.5;
-        };
-    };
-    return 0;   // BCC thinks we don't return enough
-}
-
-double Processor::defaultPriority(XSLElement *tmpl)
-{
-    Expression *e = tmpl -> getAttExpr(XSLA_MATCH);
-    if (!e) 
-        return PRIORITY_NOMATCH;
-    switch(e -> functor)
-    {
-    case EXF_LOCPATH:
-        {
-            return defaultPriorityLP(e);
-        }; break;
-    case EXFO_UNION:
-        {
-            double max=0, priority;
-            BOOL first = TRUE;
-            int eArgsNumber = e -> args.number();
-            for (int i=0; i < eArgsNumber; i++)
-            {
-                priority = defaultPriorityLP(e -> args[i]);
-                if (first || (priority > max)) 
-                    max = priority;
-                first = FALSE;
-            };
-            return max;
-        }; break;
-    default:
-        {
-            assert(!"expression not a union or LP");
-            return 0;   // dummy
-        }
-    };
-    return 0;       // dummy for BCC
-}
-
 
 /*................................................................
 getCurrentMode()
@@ -612,23 +520,35 @@ void Processor::popMode()
     modes.freelast(FALSE);
 }
 
-eFlag Processor::addLineNoTree(DataLine *&d, Str &absolute, Bool isXSL)
+eFlag Processor::addLineNoTree(Sit S, DataLine *&d, Str &absolute, Bool isXSL)
 {
-    M( situation, d = new DataLine (this) );
-    E( d -> open(absolute, DLMODE_READ) );
-    E( datalines.addLine(d, NULL, isXSL) );
+    GP( DataLine ) d_;
+    M( S, d_ = new DataLine );
+    E( (*d_).open(S, absolute, DLMODE_READ, &argList) );
+    E( datalines.addLine(S, d_, NULL, isXSL) );
+    d = d_.keep();
     return OK;
 }
 
-eFlag Processor::addLineParse(Tree *& newTree, Str &absolute, Bool isXSL)
+eFlag Processor::addLineTreeOnly(Sit S, DataLine *&d, Str &absolute, Bool isXSL,
+    Tree *t)
 {
-    DataLine *d = new DataLine (this);
-    E( d -> open(absolute, DLMODE_READ) );
-    newTree = new Tree(absolute, isXSL, this);
-    newTree -> dline = d;
-    E( parse(newTree, d));
-    E( datalines.addLine(d, newTree, isXSL) );
-    E( d -> close() );
+    GP( DataLine ) d_;
+    M( S, d_ = new DataLine );
+    E( (*d_).setURIAndClose(S, absolute) );
+    E( datalines.addLine(S, d_, t, isXSL, /* preparsedTree = */ TRUE) );
+    d = d_.keep();
+    return OK;
+}
+
+eFlag Processor::addLineParse(Sit S, Tree *& newTree, Str &absolute, Bool isXSL)
+{
+    GP( DataLine ) d = new DataLine;
+    E( (*d).open(S, absolute, DLMODE_READ, &argList) );
+    GP( Tree ) newTree_ = new Tree(absolute, isXSL );
+    E( (*newTree_).parse(S, d) );
+    E( (*d).close(S) );
+    E( datalines.addLine(S, d.keep(), newTree = newTree_.keep(), isXSL) );
     return OK;
 }
 
@@ -651,7 +571,7 @@ const Str& Processor::findBaseURI(const Str& unmappedBase)
 
 const Str& Processor::baseForVertex(Element *v)
 {
-    return findBaseURI(NZ(v) -> ownerT -> name);
+    return findBaseURI(NZ(v) -> getOwner().name);
 }
 
 
@@ -663,7 +583,7 @@ const Str& Processor::baseForVertex(Element *v)
 //  into account)
 //
 
-eFlag Processor::readTreeFromURI(Tree*& newTree, const Str& location, const Str& base,
+eFlag Processor::readTreeFromURI(Sit S, Tree*& newTree, const Str& location, const Str& base,
       Bool isXSL)
 {
     Str
@@ -671,247 +591,82 @@ eFlag Processor::readTreeFromURI(Tree*& newTree, const Str& location, const Str&
     makeAbsoluteURI(location, base, absolute);
     newTree = datalines.getTree(absolute, isXSL, DLMODE_READ);
     if (!newTree)
-        E( addLineParse(newTree, absolute, isXSL) );
+        E( addLineParse(S, newTree, absolute, isXSL) );
     return OK;
 }
 
-/*
-eFlag Processor::makeTreeForURI(Tree*& newTree, Str& location, Str& base)
+eFlag Processor::pushOutputterForURI(Sit S, Str& location, Str& base)
 {
     Str absolute;
     makeAbsoluteURI(location, base, absolute);
     if (datalines.getTree(absolute, FALSE, DLMODE_WRITE))
-        Err1(E1_CANNOT_WRITE, absolute);
-    newTree = new Tree(absolute, FALSE);
-    DataLine *d = new DataLine;
-    newTree -> dline = d;
-    E( d -> open(absolute, DLMODE_WRITE) );
-    E( datalines.addLine(d, newTree, FALSE) );
-    return OK;
-}
-*/
-
-eFlag Processor::pushOutputterForURI(Str& location, Str& base)
-{
-    Str absolute;
-    makeAbsoluteURI(location, base, absolute);
-    if (datalines.getTree(absolute, FALSE, DLMODE_WRITE))
-        Err1(situation, E1_CANNOT_WRITE, absolute);
-    DataLine *d;
-    M( situation, d = new DataLine (this) );
-    E( d -> open(absolute, DLMODE_WRITE) );
+        Err1(S, E1_URI_WRITE, absolute);
+    GP( DataLine ) d;
+    M( S, d = new DataLine );
+    E( (*d).open(S, absolute, DLMODE_WRITE, &argList) );
     // FIXME: the NULL tree in the following
-    E( datalines.addLine(d, NULL, FALSE) );
-    OutputterObj *newOut;
-    M( situation, newOut = new OutputterObj(this) );
-    E( newOut -> setOptions(d, &(styleSheet -> outputDef)) );
+    E( datalines.addLine(S, d.keep(), NULL, FALSE) );
+    GP( OutputterObj ) newOut;
+    M( S, newOut = new OutputterObj );
+    E( (*newOut).setOptions(S, d, &(styleSheet -> outputDef)) );
     if (theSAXHandler)
-        E( newOut -> setOptionsSAX(theSAXHandler, theSAXUserData) );
-    outputters_.append(newOut);
+        E( (*newOut).setOptionsSAX(S, theSAXHandler, theSAXUserData) );
+    outputters_.append(newOut.keep());
     return OK;
 }
 
-eFlag Processor::pushTreeConstructer(TreeConstructer *& newTC, Tree *t)
+eFlag Processor::pushTreeConstructer(Sit S, TreeConstructer *& newTC, Tree *t)
 {
-    M( situation, newTC = new TreeConstructer(this) );
-    E( newTC -> parseUsingSAX(t) );
-    outputters_.append(NZ(newTC -> getOutputter()));
+    newTC = NULL;
+    GP( TreeConstructer ) newTC_ = new TreeConstructer(S);
+    GP( OutputterObj ) newTCSource = new OutputterObj;
+    M( S, newTC_+0 );
+    outputters_.append(newTCSource);
+    E( (*newTC_).parseUsingSAX(S, t, *newTCSource) );
+    newTCSource.keep();
+    newTC = newTC_.keep();
     return OK;
 }
 
-eFlag Processor::pushOutputter(OutputterObj* out_)
+eFlag Processor::pushOutputter(Sit S, OutputterObj* out_)
 {
     outputters_.append(out_);
     return OK;
 }
 
-eFlag Processor::popOutputter()
+eFlag Processor::popOutputter(Sit S)
 {
     outputters_.freelast(FALSE);
     return OK;
 }
 
-eFlag Processor::popTreeConstructer(TreeConstructer *theTC)
+eFlag Processor::popTreeConstructer(Sit S, TreeConstructer *theTC)
 {
-    outputters_.deppend();
+    popOutputter(S);
     delete theTC;
     return OK;
 }
 
-
-/*****************************************************************
-findStylesheet()
-finds a xsl:stylesheet child of the given daddy. Returns NULL
-if not found.
-*****************************************************************/
-
 /*
-XSLElement *findStylesheet(Daddy &d)
+eFlag Processor::parse(Sit S, Tree *t, DataLine *d)
 {
-    Vertex *w;
-    int dContentsNumber = d.contents.number();
-    for (int i = 0; i < dContentsNumber; i++)
-    {
-        if (isXSLElement(w = d.contents[i]) && 
-            (toX(w) -> op == XSL_STYLESHEET || toX(w) -> op == XSL_TRANSFORM))
-            return toX(w);
-    };
-    return NULL;
-}
-*/
-
-Element *findStylesheet(Daddy &d)
-{
-    Vertex *w;
-    int dContentsNumber = d.contents.number();
-    for (int i = 0; i < dContentsNumber; i++)
-    {
-        if (isElement(w = d.contents[i]))
-        {
-            const QName& wName = toE(w) -> name;
-            if (!strcmp(wName.getUri(), theXSLTNamespace) && /* _PH_ */
-              ((wName.getLocal() == xslOpNames[XSL_STYLESHEET]) ||
-               (wName.getLocal() == xslOpNames[XSL_TRANSFORM])))
-            return toE(w);
-        }
-    };
-    return NULL;
-}
-
-
-Bool getWhDelimString(char *&list, Str& firstPart)
-{
-    skipWhite(list);
-    if (!*list) return FALSE;
-    char *list_was = list;
-    for(; *list && !isWhite(*list); list++);
-    firstPart.nset(list_was, (int)(list - list_was));
-    return TRUE;
-}
-
-/*****************************************************************
-Processor::processVertexAfterParse()
-Performs any necessary actions on a vertex immediately after it is
-parsed into the tree. The vertex is still 'current'. Typically, this
-function calls popVertex.
-ARGS:
-    v   the vertex to be processed
-    t   the containing tree
-The function operates only on XSL vertices, namely:
-xsl:include - replaces the vertex by the newly constructed tree
-xsl:output  - files the information into the tree's structures
-...
-Other vertices are just popped off the stack.
-*****************************************************************/
-
-eFlag Processor::processVertexAfterParse(Vertex *v, Tree *t, TreeConstructer* tc)
-{
-    XSL_OP theOp;
-    if (isXSLElement(v))
-    {
-        XSLElement *x = toX(v);
-        situation->pushCurrV(v);
-        switch(theOp = x -> op)
-        {
-        case XSL_INCLUDE:
-            {
-                Attribute *a = NZ( x -> atts.find(XSLA_HREF) );
-                Tree *srcTree;
-		        const Str& base = findBaseURI(t -> name);
-                E( readTreeFromURI(srcTree, a -> cont, base, FALSE) );
-                
-                Element *theSheet = findStylesheet(srcTree -> root);
-                if (!theSheet)
-                    Warn1(situation, W_NO_STYLESHEET, (char*)(a -> cont));
-                t -> dropCurrentElement(v);
-
-                E( tc -> parseUsingSAXForAWhile() );
-                if (theSheet)
-                    E( theSheet -> contents.copy() );
-                E( tc -> parseUsingSAXForAWhileDone() );
-            }; break;
-        case XSL_OUTPUT:
-            {
-                int i, attsNumber = x -> atts.number();
-                Attribute *theAtt;
-                for (i = 0; i < attsNumber; i++)
-                {
-                    theAtt = toA(x -> atts[i]);
-                    switch(theAtt -> op)
-                    {
-                    case XSLA_METHOD:
-                        {
-                            QName q(this);
-                            E( q.setLogical(theAtt -> cont, 
-                                &(x -> namespaces), FALSE) );
-                            E( t -> outputDef.setItemQName(XSLA_METHOD, q, TRUE) );
-                        }; break;
-                    case XSLA_CDATA_SECT_ELEMS:
-                        {
-                            QName q(this);
-                            Bool someRemains;
-                            Str listPart;
-                            char *p = theAtt -> cont;
-                            do
-                            {
-                                someRemains = getWhDelimString(p, listPart);
-                                if (someRemains)
-                                {
-                                    E( q.setLogical(listPart, 
-                                        &(x -> namespaces), TRUE) );
-                                    E( t -> outputDef.setItemQName(XSLA_CDATA_SECT_ELEMS, 
-                                        q, TRUE) );
-                                };
-                            }
-                            while (someRemains);
-                        }; break;
-                    default:
-                            E( t -> outputDef.setItemStrCheck(theAtt -> op, theAtt -> cont) );
-                    };
-                }
-            t -> popVertex();
-            }; break;
-        case XSL_NAMESPACE_ALIAS:
-          {
-             Phrase style,result;
-             dict().insert(NZ( x -> atts.find(XSLA_STYLESHEET_PREFIX)) -> cont,style);
-             dict().insert(NZ( x -> atts.find(XSLA_RESULT_PREFIX)) -> cont,result);
-             aliases.appendConstruct(style,result);
-             t -> popVertex();
-          }; break;
-        case XSL_TEMPLATE:
-            {
-                this -> insertRule(x);
-                t -> popVertex();
-            }; break;
-        default:
-            t -> popVertex();
-        }
-        situation->popCurrent();
-    }
-    else
-        t -> popVertex();
-    return OK;
-}
-
-eFlag Processor::parse(Tree *t, DataLine *d)
-{
-    Log1(situation, L1_PARSING, t -> name);
+    Log1(S, L1_PARSING, t -> name);
     double time_was = getMillisecs();
-    TreeConstructer tc(this);
-    eFlag retval = tc.parseDataLineUsingExpat(t, d);
+    TreeConstructer tc(S);
+    eFlag retval = tc.parseDataLineUsingExpat(S, t, d);
     if (!retval)
     {
-        Log1(situation, L1_PARSE_DONE, getMillisecsDiff(time_was));
+        Log1(S, L1_PARSE_DONE, getMillisecsDiff(time_was));
     }
     return retval;
 }
+*/
 
-eFlag Processor::getArg(const char* name, char*& buffer, Bool isUTF16)
+eFlag Processor::getArg(Sit S, const char* name, char*& buffer, Bool isUTF16)
 {
     Str temp, *value = argList.find(temp = (char*)name);
     if (!value)
-        Err1(situation, E1_ARG_NOT_FOUND,(char*) name);
+        Err1(S, E1_ARG_NOT_FOUND,(char*) name);
     buffer = (char*) *value;
     return OK;
 }
@@ -924,7 +679,7 @@ eFlag Processor::getArg(const char* name, char*& buffer, Bool isUTF16)
 //
 //
 
-eFlag Processor::setHandler(HandlerType type, void *handler, void *userData)
+eFlag Processor::setHandler(Sit S, HandlerType type, void *handler, void *userData)
 {
     void **whereHandler, **whereUserData;
     switch(type)
@@ -949,13 +704,18 @@ eFlag Processor::setHandler(HandlerType type, void *handler, void *userData)
             whereHandler = (void **)&theMiscHandler;
             whereUserData = &theMiscUserData;
         }; break;
+    case HLR_ENC:
+        {
+            whereHandler = (void **)&theEncHandler;
+            whereUserData = &theEncUserData;
+        }; break;
     default: 
-        Err1(situation, E1_INVALID_HLR_TYPE, (int) type);
+        Err1(S, E1_INVALID_HLR_TYPE, (int) type);
     }
     if (*whereHandler)
     {
         if (handler)
-            Warn1(situation, W1_HLR_REGISTERED, hlrTypeNames[type])
+            Warn1(S, W1_HLR_REGISTERED, hlrTypeNames[type])
         else
         {
             *whereHandler = NULL;
@@ -970,7 +730,7 @@ eFlag Processor::setHandler(HandlerType type, void *handler, void *userData)
             *whereUserData = userData;
         }
         else
-            Warn1(situation, W1_HLR_NOT_REGISTERED, hlrTypeNames[type])
+            Warn1(S, W1_HLR_NOT_REGISTERED, hlrTypeNames[type])
     }
     return OK;
 }
@@ -1004,6 +764,13 @@ MiscHandler *Processor::getMiscHandler(void **udata)
     return theMiscHandler;
 }
 
+EncHandler *Processor::getEncHandler(void **udata)
+{
+    if (udata)
+        *udata = theEncUserData;
+    return theEncHandler;
+}
+
 void *Processor::getHandlerUserData(HandlerType type, void *handler)
 {
     switch(type)
@@ -1025,7 +792,6 @@ const Str& Processor::getHardEncoding() const
     return hardEncoding;
 };
 
-
 /*****************************************************************
 copyArg
   called if the result document's location uses the arg:
@@ -1053,14 +819,14 @@ void Processor::copyArg(const Str& argName, int* argOrdinal,
         return;
     }
     DynBlock *block = NZ( datalines[lineNo] -> _dataline -> getOutBuffer() );
-    newCopy = block -> compactToBuffer();
+    newCopy = block -> compactToBuffer(); // GP: OK (no exit route)
 
     //  set *argOrdinal
     *argOrdinal = argList.findNum((char *)absolute + 4);    // skip 'arg:'
 
 }
 
-eFlag Processor::useArg(char *name, char *val)
+eFlag Processor::useArg(Sit S, char *name, char *val)
 {
     assert(name);
     DStr nameStr;
@@ -1068,7 +834,7 @@ eFlag Processor::useArg(char *name, char *val)
         nameStr = "/";
     nameStr += name;
     if (argList.find(nameStr))
-        Err1(situation, E1_DUPLICATE_ARG, nameStr);
+        Err1(S, E1_DUPLICATE_ARG, nameStr);
     StrStr *p = new StrStr;
     p -> key = nameStr;
     if (val)
@@ -1076,17 +842,55 @@ eFlag Processor::useArg(char *name, char *val)
     else
         p -> value.empty();
     argList.append(p);
+    addedFlag = TRUE;
     return OK;
 }
 
-eFlag Processor::useGlobalParam(char *name, char *val)
+eFlag Processor::useTree(Sit S, char *name, Tree *t)
 {
     assert(name);
-    QName q(this);
-    q.setLocal(name);
-    Expression *expr = new Expression(NULL,this,EXF_ATOM);
+    DStr nameStr;
+    if (*name != '/')
+        nameStr = "/";
+    nameStr += name;
+	// to check for duplicate trees
+	E( useArg(S, name, NULL) );
+	
+	Str absolute;
+	DataLine *d;
+	makeAbsoluteURI(nameStr, "arg:", absolute);
+	E( addLineTreeOnly(S, d, absolute, t -> XSLTree, t) );
+    addedFlag = TRUE;
+    return OK;
+}
+
+eFlag Processor::addGlobalParam(Sit S, char *name, char *val)
+{
+    assert(name);
+    if (!val) val = (char*)"";
+    globalParamsList.appendConstruct(name, val);
+    return OK;
+}
+
+eFlag Processor::useGlobalParam(Sit S, char *name, char *val)
+{
+    assert(name);
+    QName q;
+    q.setLocal(NZ(styleSheet) -> unexpand(name));
+    Expression *expr = new Expression(styleSheet -> getRoot(), EXF_ATOM);
     expr -> setAtom(val);
-    vars -> addPrebinding(q, expr);
+    vars -> addPrebinding(S, q, expr);
+    return OK;
+}
+
+eFlag Processor::useGlobalParams(Sit S)
+{
+    while (globalParamsList.number())
+    {
+        StrStr& item = *(globalParamsList.last());
+        E( useGlobalParam(S, item.key, item.value) );
+	    globalParamsList.freelast(FALSE);	    
+    }
     return OK;
 }
 
@@ -1104,10 +908,11 @@ void Processor::addBaseURIMapping(const Str& scheme, const Str& mapping)
         baseURIMappings.appendConstruct(scheme, mapping);
 }
 
-eFlag Processor::freeResultArgs()
+eFlag Processor::freeResultArgs(Sit S)
 {
     datalines.freeall(FALSE);
     argList.freeall(FALSE);
+    addedFlag = FALSE;
     return OK;
 }
 
@@ -1116,7 +921,7 @@ Arena *Processor::getArena()
     return &theArena;
 }
 
-Str Processor::getAliasedName(const QName& name, NamespaceStack& currNamespaces)
+Str Processor::getAliasedName(const EQName& name, NamespaceStack& currNamespaces)
 {
    DStr s;
 
@@ -1125,19 +930,27 @@ Str Processor::getAliasedName(const QName& name, NamespaceStack& currNamespaces)
       Phrase newPrefix = UNDEF_PHRASE;
       Str myUri = name.getUri();
       int i;
-      for (i = 0; i < aliases.number(); i++)
+      if (styleSheet)
       {
-        if (*currNamespaces.getUri(dictionary.getKey(aliases[i]->key))
-            == myUri)
-        {
-           newPrefix = aliases[i]->value;
-        };
+          for (i = 0; i < styleSheet -> aliases().number(); i++)
+          {
+	          const Str* aliasUri = 
+		          currNamespaces.getUri(styleSheet -> expand(
+			      styleSheet -> aliases()[i]->key));
+              if (aliasUri && *aliasUri == myUri)
+              {
+                  newPrefix = styleSheet -> aliases()[i] -> value;
+              };	        
+	      }
       };
+      
       if (newPrefix == UNDEF_PHRASE)
       {
         s += name.getPrefix();
-      } else {
-        Str newPrefixStr = dictionary.getKey(newPrefix);
+      } 
+      else 
+      {
+        Str newPrefixStr = styleSheet -> expand(newPrefix);
         s += newPrefixStr;
         currNamespaces.appendConstruct(name.getPrefix(),name.getUri(),TRUE);
         currNamespaces.appendConstruct(newPrefixStr,
@@ -1149,3 +962,10 @@ Str Processor::getAliasedName(const QName& name, NamespaceStack& currNamespaces)
    s += name.getLocal();
    return s;
 }
+
+void Processor::report(Sit S, MsgType type, MsgCode code, 
+    const Str &arg1, const Str &arg2) const
+{
+    S.message(type, code, arg1, arg2);
+}
+
