@@ -33,8 +33,9 @@
 #ifndef ProcHIncl
 #define ProcHIncl
 
+// GP: clean
+
 #define PROC_ARENA_SIZE         0x10000
-#define PROC_DICTIONARY_LOGSIZE    10
 
 enum StandardPhrase
 {
@@ -52,6 +53,7 @@ enum StandardPhrase
 #include "datastr.h"
 #include "uri.h"
 #include "output.h"
+#include "encoding.h"
 // #include "vars.h"
 //#include "expr.h"
 //#include "context.h"
@@ -98,11 +100,10 @@ class Processor;
 class RuleSList : public SList<RuleItem*>
 {
 public:
-    RuleSList(Processor* proc);
+    RuleSList();
     ~RuleSList();
     int compare(int first, int second);
-    XSLElement *findByName(QName &);
-    Processor* proc;
+    XSLElement *findByName(const Tree &t, const QName &q) const;
 };
 
 /*****************************************************************
@@ -114,10 +115,14 @@ DataLinesList
 class DataLineItem
 {
 public:
+    DataLineItem(Sit S_);
     ~DataLineItem();
     DataLine *_dataline;
     Tree *_tree;
     Bool _isXSL;
+    Bool _preparsedTree;
+private:
+    Sit situation;
 };
 
 class DataLinesList: public PList<DataLineItem*>
@@ -126,7 +131,9 @@ public:
     int findNum(Str &absoluteURI, Bool _isXSL,
         DLAccessMode _mode);
     Tree *getTree(Str &absoluteURI, Bool _isXSL, DLAccessMode _mode);
-    eFlag addLine(DataLine *d, Tree *t, Bool isXSL);
+    eFlag addLine(Sit S, DataLine *d, Tree *t, Bool isXSL,
+        Bool preparsedTree = FALSE);
+	
 };
 
 
@@ -143,34 +150,27 @@ class Element;
 class Processor /* : public SabObj */
 {
 public:
-    SituationObj situation_;
-    SituationObj* situation;
     Tree *input,
         *styleSheet;
-//        *result;
-    RuleSList rules;
     Processor();
     ~Processor();
-    eFlag open(char *sheetURI, char *inputURI, char *resultURI);
-    eFlag execute(Vertex *, Context *);
-    eFlag execute(VertexList&, Context *);
-    eFlag parse(Tree *, DataLine *d);
-    double defaultPriorityLP(Expression *);
-    double defaultPriority(XSLElement *);
-    eFlag bestRule(XSLElement *&, Context *);
-    eFlag builtinRule(Context *c);
-    eFlag insertRule(XSLElement *);
+    eFlag open(Sit S, char *sheetURI, char *inputURI);
+    eFlag run(Sit S, char* resultURI);
+    eFlag execute(Sit S, Vertex *, Context *&);
+    eFlag execute(Sit S, VertexList&, Context *&);
+    // moving to Tree:
+//    eFlag parse(Sit S, Tree *, DataLine *d);
+    eFlag bestRule(Sit S, XSLElement *&, Context *);
+    eFlag builtinRule(Sit S, Context *c);
     FILE *logfile;
     unsigned long allocObjects,
         allocBytes,
         maxAllocObjects,
         maxAllocBytes;
     Expression* getVarBinding(QName &);
-    Bool cmpQNames(const QName&, const QName&) const;
     VarsList *vars;
     OutputterObj *outputter()
     {
-//        assert(outputters_.number()); 
         if (outputters_.number())
             return outputters_.last();
         else return NULL;
@@ -178,33 +178,38 @@ public:
     QName *getCurrentMode();
     void pushMode(QName *);
     void popMode();
-    eFlag readTreeFromURI(Tree*& newTree, const Str& location, const Str& base, Bool isXSL);
-    eFlag pushOutputterForURI(Str& location, Str& base);
-    eFlag pushTreeConstructer(TreeConstructer *&newTC, Tree *t);
-    eFlag pushOutputter(OutputterObj* out_);
-    eFlag popOutputter();
-    eFlag popTreeConstructer(TreeConstructer *theTC);
-    // eFlag makeTreeForURI(Tree*& newTree, Str& location, Str& base);
-    eFlag processVertexAfterParse(Vertex *, Tree *, TreeConstructer *tc);
+    eFlag readTreeFromURI(Sit S, Tree*& newTree, const Str& location, const Str& base, Bool isXSL);
+    eFlag pushOutputterForURI(Sit S, Str& location, Str& base);
+    eFlag pushTreeConstructer(Sit S, TreeConstructer *&newTC, Tree *t);
+    eFlag pushOutputter(Sit S, OutputterObj* out_);
+    eFlag popOutputter(Sit S);
+    eFlag popTreeConstructer(Sit S, TreeConstructer *theTC);
+    eFlag processVertexAfterParse(Sit S, Vertex *, Tree *, TreeConstructer *tc);
 
-    eFlag setHandler(HandlerType type, void *handler, void *userData);
+    eFlag setHandler(Sit S, HandlerType type, void *handler, void *userData);
     SchemeHandler *getSchemeHandler(void **udata);
     MessageHandler *getMessageHandler(void **udata);
     SAXHandler *getSAXHandler(void **udata);
     MiscHandler *getMiscHandler(void **udata);
+    EncHandler *getEncHandler(void **udata);
 
     void *getHandlerUserData(HandlerType type, void *handler);
 
     void setHardEncoding(const Str& hardEncoding_);
     const Str& getHardEncoding() const;
 
-    eFlag addLineNoTree(DataLine *&d, Str &absolute, Bool isXSL);
-    eFlag addLineParse(Tree *& newTree, Str &absolute, Bool isXSL);
+    eFlag addLineNoTree(Sit S, DataLine *&d, Str &absolute, Bool isXSL);
+    eFlag addLineParse(Sit S, Tree *& newTree, Str &absolute, Bool isXSL);
+    eFlag addLineTreeOnly(Sit S, DataLine *&d, Str &absolute, Bool isXSL,
+        Tree *t);
     void copyArg(const Str& argName, int* argOrdinal, char*& newCopy);
-    eFlag getArg(const char*, char*&, Bool);
-    eFlag useArg(char *name, char *val);
-    eFlag freeResultArgs();
-    eFlag useGlobalParam(char *name, char *val);
+    eFlag getArg(Sit S, const char*, char*&, Bool);
+    eFlag useArg(Sit S, char *name, char *val);
+	eFlag useTree(Sit S, char *name, Tree *t);
+    eFlag freeResultArgs(Sit S);
+    eFlag addGlobalParam(Sit S, char *name, char *val);
+    eFlag useGlobalParam(Sit S, char *name, char *val);
+    eFlag useGlobalParams(Sit S);
     const Str& baseForVertex(Element *v);
     const Str& findBaseURI(const Str& unmappedBase);
     void addBaseURIMapping(const Str& scheme, const Str& mapping);
@@ -219,18 +224,28 @@ public:
     { 
         return instanceData; 
     };
+    
+    void rememberSituation(Situation *SP)
+    {
+        instanceSituation = SP;
+    }
+    
+    Situation* recallSituation()
+    {
+        return instanceSituation;
+    }
+    
     void prepareForRun();
-    void cleanupAfterRun();
+    void cleanupAfterRun(Sit S);
     const QName theEmptyQName;
     Arena *getArena();
-    HashTable& dict() { return dictionary; }
-    Phrase stdPhrase(StandardPhrase s) const { assert(s <= PHRASE_LAST); return stdPhrases[s]; }
-    Str getAliasedName(const QName& name, NamespaceStack& currNamespaces);
-
+    Str getAliasedName(const EQName& name, NamespaceStack& currNamespaces);
+    void report(Sit S, MsgType type, MsgCode code, 
+		const Str &arg1, const Str &arg2) const;
+    Bool getAddedFlag() {return addedFlag;}
 private:
-    P2List aliases;
-    eFlag openCommon();
-    eFlag execApply(Context *c);
+    eFlag openCommon(Sit S);
+    eFlag execApply(Sit S, Context *c);
     PList<QName*> modes;
     StrStrList
         argList;
@@ -241,18 +256,20 @@ private:
     MessageHandler *theMessageHandler;
     SAXHandler *theSAXHandler;
     MiscHandler *theMiscHandler;
+    EncHandler *theEncHandler;
     void 
         *theSchemeUserData,
         *theMessageUserData,
         *theSAXUserData,    
-        *theMiscUserData;    
+        *theMiscUserData,
+        *theEncUserData;    
     void *instanceData;
     void freeNonArgDatalines();
-    StrStrList baseURIMappings;
-    Arena theArena;
-    HashTable dictionary;
-    Phrase stdPhrases[PHRASE_LAST];
+    StrStrList baseURIMappings, globalParamsList;
     Str hardEncoding;
+    Arena theArena;
+    Situation *instanceSituation;
+    Bool addedFlag;
 };
 
 #endif //ifndef ProcHIncl
