@@ -36,7 +36,10 @@ i n c l u d e s
 *                                       *
 ****************************************/
 
-#include<iostream.h>
+// #include<iostream.h>
+
+// GP: clean
+
 #include <stdarg.h>
 
 #define SablotAsExport
@@ -48,6 +51,9 @@ i n c l u d e s
 #include "expr.h"
 #include "proc.h"
 #include "situa.h"
+#include "uri.h"
+#include "parser.h"
+#include "guard.h"
 
 //
 //  defines
@@ -55,6 +61,12 @@ i n c l u d e s
 
 #define P( PTR ) ((Processor *)(PTR))
 #define EE(statement) {int code___=(statement); if (code___) return code___;}
+// same as EE() but destroys the given processor
+#define EE_D(CODE,PROC) {int code___=(CODE); if (code___) {SablotDestroyProcessor(PROC); return code___;}}
+// error-checking chain
+#define EC( VAR, STMT ) {VAR = VAR || (STMT);}
+#define SIT( S ) (*(Situation*)S)
+
 
 /****************************************
 g l o b a l s
@@ -67,22 +79,163 @@ g l o b a l s
 d e f i n i t i o n s
 ****************************************/
 
-void doStart(SituationObj* situation)
+void doStart(Sit S)
 {
-    Log1(situation, L_START, situation->timeStr());
+    S.message(MT_LOG, L_START, S.timeStr(), (char*) NULL);
+//    Log1(S, L_START, S.timeStr());
 };
 
-void doEnd(SituationObj* situation)
+void doEnd(Sit S)
 {
-    Log1(situation, L_STOP, situation->timeStr());
+    S.message(MT_LOG, L_STOP, S.timeStr(), (char*) NULL);
+//    Log1(S, L_STOP, S.timeStr());
 }
 
-#define ErrQ(situation, e) {\
-    if (situation) { \
-        situation->error(e,""/**theEmptyString*/,""/**theEmptyString*/); \
-        doEnd(situation); \
-        return e; \
-    } \
+#define ErrQ(S, e) {\
+    S.message(MT_ERROR, e,(char*) NULL, (char*)NULL); \
+    doEnd(S); \
+    return e; \
+	}
+
+// report function to enable the Sablot... globals to use Err and Log		
+void report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2) 
+{
+    S.message(type, code, arg1, arg2);
+}
+
+
+//
+//
+//      situation functions
+//
+//
+
+int SablotCreateSituation(SablotSituation *sPtr)
+{
+    *sPtr = new Situation;
+    if (!*sPtr)
+    {
+        return NOT_OK;
+	}
+    return OK;
+}
+
+int SablotDestroySituation(SablotSituation S)
+{
+    if (S) delete (Situation*)S;
+    S = NULL;
+    return OK;
+}
+
+int SablotSetOptions(SablotSituation S, int flags)
+{
+    ((Situation*)S) -> setFlags(flags);
+    return OK;
+}
+
+int SablotClearSituation(SablotSituation S)
+{
+    ((Situation*)S) -> clear();
+    return OK;
+}
+
+//
+//    document functions
+//
+
+int SablotCreateDocument(SablotSituation S, SDOM_Document *D)
+{
+    // non-XSL tree with empty name
+    Tree *t = new Tree((char*)"", FALSE);
+    *D = &(t -> getRoot());
+    return OK;
+}
+
+int SablotParse_(
+    SablotSituation S, 
+    const char *uri, 
+    const char *buffer,
+    SDOM_Document *D,
+    Bool asStylesheet)
+{
+    Str absolute;
+    StrStrList argList;
+    DStr base;
+    *D = NULL;
+    double time_was = getMillisecs();
+    // make absolute address
+    if (!buffer)
+    {
+        my_getcwd(base);
+        base = Str("file://") + base;
+        // should call Processor::findBaseURI() here (but have no Processor)
+        makeAbsoluteURI(uri, base, absolute);
+	}
+	else
+	    absolute = "arg:/_parsed_";
+    
+    // create the tree and dataline
+    DataLine line;
+    GP( Tree ) newTree = new Tree(absolute, /* isXSL = */ asStylesheet);
+    TreeConstructer tc(SIT(S));
+    
+    // add buffer if there is one
+    if (buffer)
+        argList.appendConstruct("/_parsed_", buffer);
+		    
+    // open the line
+    EE( line.open(SIT(S), absolute, DLMODE_READ, &argList) );
+    Log1(SIT(S), L1_PARSING, absolute);
+    EE( tc.parseDataLineUsingExpat(SIT(S), newTree, &line) );        
+    EE( line.close(SIT(S)) );
+    *D = &(newTree.keep() -> getRoot());    
+    Log1(SIT(S), L1_PARSE_DONE, getMillisecsDiff(time_was));
+    return OK;
+}
+
+
+int SablotParse(
+    SablotSituation S, 
+    const char *uri, 
+    SDOM_Document *D)
+{
+    return SablotParse_(S, uri, NULL, D, FALSE);
+}
+
+int SablotParseBuffer(
+    SablotSituation S, 
+    const char *buffer, 
+    SDOM_Document *D)
+{
+    return SablotParse_(S, NULL, buffer, D, FALSE);
+}
+
+int SablotParseStylesheet(
+    SablotSituation S, 
+    const char *uri, 
+    SDOM_Document *D)
+{
+    return SablotParse_(S, uri, NULL, D, TRUE);
+}
+
+int SablotParseStylesheetBuffer(
+    SablotSituation S, 
+    const char *buffer, 
+    SDOM_Document *D)
+{
+    return SablotParse_(S, NULL, buffer, D, TRUE);
+}
+
+int SablotLockDocument(SablotSituation S, SDOM_Document D)
+{
+    ((RootNode*)D) -> getOwner().makeStamps();
+    return OK;
+}
+
+int SablotDestroyDocument(SablotSituation S, SDOM_Document D)
+{
+    delete &((toRoot(toV(D))) -> getOwner());
+    return OK;
 }
 
 
@@ -96,21 +249,122 @@ int SablotCreateProcessor(void **processorPtr)
 {
     // to debug memory leaks in MSVC, uncomment the call to checkLeak()
     // and set _crtBreakAlloc to the # of block (proc=51)
+    void *newSit = NULL;
+    
+//    SablotCreateSituation(&((void*)newSit));    
+    SablotCreateSituation(&newSit);
 #if defined(HAVE_MTRACE) && defined(CHECK_LEAKS)
     mtrace();
 #endif
     *processorPtr = new Processor;
     if (!*processorPtr)
     {
-        SituationObj* situation = NULL;
-        ErrQ( situation, E_MEMORY );
+        ((Situation*)newSit) -> message(MT_ERROR, E_MEMORY, (char*) NULL, (char*) NULL);
+        // Err( *newSit, E_MEMORY );
     }
+    // the processor must remember its situation (this is just to
+    // avoid compatibility breakdown)
+    P( *processorPtr ) -> rememberSituation((Situation*)newSit);
+    ((Situation*)newSit) -> setProcessor( P(*processorPtr) );
 
-    checkErr( P(*processorPtr)->situation );
-    doStart ( P(*processorPtr)->situation );
+    // the following seems superfluous - the processor is new
+    // checkErr( P(*processorPtr)->situation );
+    doStart ( *(Situation*)newSit );
     return OK;
 }
 
+int SablotAddParam(
+    SablotSituation S,
+    void *processor_,
+    const char *paramName,
+    const char *paramValue)
+{
+    void *saveproc = processor_;
+    // using the situation passed
+    // Sit S = *(NZ(P( processor_ )) -> recallSituation());
+    SIT(S).swapProcessor(saveproc);    
+    SIT(S).clear();
+    int code = P(processor_) -> addGlobalParam(SIT(S), 
+        (char*)paramName, (char*)paramValue);
+    //SIT(S).swapProcessor(processor_);
+    SIT(S).swapProcessor(saveproc);
+    return code;	
+}
+
+int SablotAddArgBuffer(
+    SablotSituation S,
+    void *processor_,
+    const char *argName,
+    const char *bufferValue)
+{
+    void *saveproc = processor_;
+    // using the situation passed
+    // Sit S = *(NZ(P( processor_ )) -> recallSituation());
+    SIT(S).swapProcessor(saveproc);    
+    SIT(S).clear();
+    if (!(P(processor_) -> getAddedFlag()))
+        SablotFreeResultArgs (processor_);
+    int code = P(processor_) -> useArg(SIT(S), (char*)argName, (char*)bufferValue);
+    SIT(S).swapProcessor(saveproc);
+    return code;    
+}
+
+int SablotAddArgTree(
+    SablotSituation S,
+    void *processor_,
+    const char *argName,
+    SDOM_Document tree)
+{
+    void *saveproc = processor_;
+    // using the situation passed
+    // Sit S = *(NZ(P( processor_ )) -> recallSituation());
+    SIT(S).swapProcessor(saveproc);
+    SIT(S).clear();
+    if (!(P(processor_) -> getAddedFlag()))
+        SablotFreeResultArgs (processor_);
+    int code = P(processor_) -> useTree(SIT(S), (char*) argName, 
+        &(toRoot(tree) -> getOwner()));
+    SIT(S).swapProcessor(saveproc);
+    return code;
+}
+
+int SablotRunProcessorGen(
+    SablotSituation S,
+    void *processor_,
+    char *sheetURI, 
+    char *inputURI, 
+    char *resultURI)
+{
+    void *saveproc = processor_;
+    // using the situation passed
+    // Sit S = *(NZ(P( processor_ )) -> recallSituation());
+    int code = 0;
+    SIT(S).swapProcessor(saveproc);
+    SIT(S).clear();
+    // remove the existing 'arg' buffers
+    if (!(P(processor_) -> getAddedFlag()))
+        EC( code, SablotFreeResultArgs (processor_) );
+    if (!code)
+        P(processor_) -> prepareForRun();
+    // must first open the files, then use global params
+    // because VarsList needs stylesheet's dictionary
+    EC( code, P(processor_) -> open(SIT(S), sheetURI,inputURI) );
+    EC( code, P(processor_) -> useGlobalParams(SIT(S)) );
+    EC( code, P(processor_) -> run(SIT(S), resultURI) );
+    P(processor_) -> cleanupAfterRun(SIT(S));
+    if (code)
+    {
+        // GP: if not all files could be opened, those that could are
+        // closed in cleanupAfterRun
+        code = SIT(S).getError();
+        P(processor_) -> freeResultArgs(SIT(S));
+    }
+    else
+        code = OK;
+    SIT(S).swapProcessor(saveproc);	
+    return code;
+}
+
 
 //
 //
@@ -123,35 +377,45 @@ int SablotRunProcessor(void *processor_,
                        char *resultURI,
                        char **params, 
                        char **arguments)
-{
+{    
     Bool problem = FALSE;
+    Sit S = *(NZ(P( processor_ )) -> recallSituation());
 
     {
         // remove the existing 'arg' buffers
+	    S.clearError();
         E( SablotFreeResultArgs (processor_));
         P(processor_) -> prepareForRun();
         char **p;
         if (arguments)
         {
             for (p = arguments; *p && !problem; p += 2)
-                problem = P(processor_) -> useArg(p[0], p[1]);
+                problem = P(processor_) -> useArg(S, p[0], p[1]);
         }
-        if (params)
+	    if (!problem)
+	    {
+	        // must first open the files, then use global params
+		    // because VarsList needs stylesheet's dictionary
+	        problem = P(processor_) -> open(S, sheetURI,inputURI);
+            if (params)
+            {
+                for (p = params; *p && !problem; p += 2)
+                    problem = P(processor_) -> useGlobalParam(S, p[0], p[1]);
+            }
+	    }
+        if (problem || P(processor_) -> run(S, resultURI))
         {
-            for (p = params; *p && !problem; p += 2)
-                problem = P(processor_) -> useGlobalParam(p[0], p[1]);
-        }
-        if (problem || P(processor_) -> open(sheetURI,inputURI,resultURI))
-        {
-            int code = P(processor_) -> situation -> getError();
-            P(processor_) -> cleanupAfterRun();
-            P(processor_) -> freeResultArgs();
+            // GP: if not all files could be opened, those that could are
+            // closed in cleanupAfterRun
+            int code = S.getError();
+            P(processor_) -> cleanupAfterRun(S);
+            P(processor_) -> freeResultArgs(S);
 //            cdelete(proc);
-            P(processor_) -> situation->clear();
+            // S.clear(); (why forget the message?)
             return code;
         }
     }
-    P(processor_) -> cleanupAfterRun();
+    P(processor_) -> cleanupAfterRun(S);
     return OK;
 }
 
@@ -166,16 +430,19 @@ int SablotRunProcessor(void *processor_,
 
 int SablotDestroyProcessor(void *processor_)
 {
-    Processor *proc = P( processor_ );
-    proc ->situation->clear();
-    doEnd(proc -> situation);
-    EE( SablotFreeResultArgs (processor_));
-    cdelete( proc );
+    int code;
+    Situation *SP = P(processor_) -> recallSituation();
+    code = SablotFreeResultArgs(processor_);
+    doEnd(*SP);
+    if(processor_) delete P( processor_);
+    processor_ = NULL;
+    
+    cdelete(SP);
 #if defined(WIN32) && defined(CHECK_LEAKS)
     memStats();
     checkLeak();
 #endif
-    return OK;
+    return code;
 };
 
 
@@ -220,7 +487,7 @@ int SablotGetResultArg(void *processor_,
                        char *argURI,
                        char **argValue)
 {
-    char *newCopy;
+    char *newCopy; // GP: OK
     // we even obtain the index of the output buffer but won't use it
     int resultNdx_;
     if (argValue)
@@ -242,7 +509,7 @@ int SablotGetResultArg(void *processor_,
 int SablotSetLog(void *processor_,
     const char *logFilename, int logLevel)
 {
-    (P(processor_) -> situation)->msgOutputFile((char *)"stderr",(char*) logFilename);
+    P(processor_) -> recallSituation()->msgOutputFile((char *)"stderr",(char*) logFilename);
     // logLevel ignored so far
     return OK;
 }
@@ -272,16 +539,15 @@ RETURNS
                 point to it (otherwise to NULL). Free it using SablotFree().
 *****************************************************************/
 
-
 int SablotProcess(char *sheetURI, char *inputURI, char *resultURI,
     char **params, char **arguments, char **resultArg)
 {
     void *theproc;
     EE( SablotCreateProcessor (&theproc) );
-    EE( SablotRunProcessor(theproc,
+    EE_D( SablotRunProcessor(theproc,
         sheetURI, inputURI, resultURI,
-        params, arguments) );
-    EE( SablotGetResultArg(theproc, resultURI, resultArg) );
+        params, arguments), theproc );
+    EE_D( SablotGetResultArg(theproc, resultURI, resultArg), theproc );
     EE( SablotDestroyProcessor(theproc) );
     return OK;
 };
@@ -338,9 +604,9 @@ int SablotProcessStrings(char *styleSheetStr,
     NULL
   };
   return SablotProcess((char *) "arg:/_stylesheet",
-		       (char *) "arg:/_xmlinput",
-		       (char *) "arg:/_output",
-		       NULL, argums, resultStr);
+                       (char *) "arg:/_xmlinput",
+                       (char *) "arg:/_output",
+                       NULL, argums, resultStr);
 };
 
 
@@ -375,14 +641,14 @@ int SablotProcessStringsWithBase(char *styleSheetStr,
     };
     void *theproc;
     EE( SablotCreateProcessor(&theproc));
-    EE( SablotSetBase(theproc, theHardBase));
-    EE( SablotRunProcessor(theproc,
+    EE_D( SablotSetBase(theproc, theHardBase), theproc );
+    EE_D( SablotRunProcessor(theproc,
         (char *) "arg:/_stylesheet",
-	(char *) "arg:/_xmlinput",
-	(char *) "arg:/_output",
-        NULL, argums));
-    EE( SablotGetResultArg(theproc,
-        (char *) "arg:/_output", resultStr));
+        (char *) "arg:/_xmlinput",
+        (char *) "arg:/_output",
+        NULL, argums), theproc);
+    EE_D( SablotGetResultArg(theproc,
+        (char *) "arg:/_output", resultStr), theproc);
     EE( SablotDestroyProcessor(theproc));
     return OK;
 };
@@ -412,7 +678,7 @@ int SablotFree(char *resultStr)
 
 int SablotFreeResultArgs(void *processor_)
 {
-    EE( P(processor_) -> freeResultArgs() );
+    EE( P(processor_) -> freeResultArgs(*(P(processor_) -> recallSituation())) );
     return OK;
 }
 
@@ -429,9 +695,10 @@ int SablotRegHandler(void *processor_,
                      void *handler, 
                      void *userData)
 {
-    EE( P(processor_) -> setHandler(type, handler, userData) );
+    Sit S = *(P(processor_) -> recallSituation());
+    EE( P(processor_) -> setHandler(S, type, handler, userData) );
     if (type == HLR_MESSAGE)
-        EE( P(processor_) -> situation -> closeFiles() );
+        EE( S.closeFiles() );
     return OK;
 }
 
@@ -448,9 +715,10 @@ int SablotUnregHandler(void *processor_,
                        void *handler,
                        void *userData)
 {
-    EE( P(processor_) -> setHandler(type, NULL, NULL) );
+    Sit S = *(P(processor_) -> recallSituation());
+    EE( P(processor_) -> setHandler(S, type, NULL, NULL) );
     if (type == HLR_MESSAGE)
-        EE( P(processor_) -> situation -> openDefaultFiles() );
+        EE( S.openDefaultFiles() );
     return OK;
 }
 
@@ -474,7 +742,7 @@ const char *SablotGetMsgText(int code)
 
 int SablotClearError(void *processor_)
 {
-    P(processor_) -> situation -> clear();
+    P(processor_) -> recallSituation() -> clear();
     return OK;
 }
 
