@@ -33,6 +33,10 @@
 #ifndef ShandlerHIncl
 #define ShandlerHIncl
 
+#include <stddef.h>
+
+/* GP: clean */
+
 /*****************************************************************
 
   handler types
@@ -44,7 +48,8 @@ typedef enum
     HLR_MESSAGE = 0,
     HLR_SCHEME,
     HLR_SAX,
-    HLR_MISC
+    HLR_MISC,
+    HLR_ENC
 } HandlerType;
 
 extern const char* hlrTypeNames[]; /* found in base.cpp */
@@ -294,4 +299,68 @@ typedef struct
     MiscHandlerDocumentInfo     *documentInfo;
 } MiscHandler;
 
+/*****************************************************************
+EncHandler
+
+  Handler for recoding requests in absence of iconv.
+*****************************************************************/
+
+#define EH_FROM_UTF8 1
+#define EH_TO_UTF8 0
+
+/*
+    the conversion descriptor like iconv_t
+*/
+
+typedef void* EHDescriptor;
+
+typedef enum 
+{
+    EH_OK,
+    EH_EINVAL,
+    EH_E2BIG,
+    EH_EILSEQ
+} EHResult;
+
+/*
+    open()
+    direction is either EH_FROM_UTF8 or EH_TO_UTF8
+    encoding is the other encoding
+    RETURN the descriptor, or -1 if the encoding is not supported
+*/
+
+typedef EHDescriptor EncHandlerOpen(void* userData, SablotHandle processor_,
+    int direction, const char *encoding);
+
+/*
+    conv()
+    arguments 3 through 7 are just like for iconv, see the manpage
+    RETURN -1 on error (set errno), a different value (e.g. 0) if OK
+*/
+
+typedef EHResult EncHandlerConv(void* userData, SablotHandle processor_,
+    EHDescriptor cd, const char** inbuf, size_t *inbytesleft,
+    char ** outbuf, size_t *outbytesleft);
+
+/*
+    close()
+    cd is the descriptor to close. Return 0 if OK, -1 on error.
+*/
+
+typedef int EncHandlerClose(void* userData, SablotHandle processor_,
+    EHDescriptor cd);
+
+/*
+    The EncHandler structure. 
+    Use SablotRegHandler(HLR_ENC, ...) to register.
+*/
+
+typedef struct
+{
+    EncHandlerOpen      *open;
+    EncHandlerConv      *conv;
+    EncHandlerClose     *close;
+} EncHandler;
+
+
 #endif
