@@ -33,6 +33,8 @@
 #ifndef TreeHIncl
 #define TreeHIncl
 
+// GP: clean
+
 #include "base.h"
 #include "verts.h"
 #include "datastr.h"
@@ -52,44 +54,76 @@ public:
 };
 */
 
+class RootNode;
+
+#define TREE_ARENA_SIZE         0x10000
+#define TREE_DICT_LOGSIZE    10
+
+
 /****************************************
 T r e e
 ****************************************/
 
-class Tree;
-
-class Tree /* : public SabObj */
+class Tree
 {
 public:
-    Tree(const Str&, BOOL, Processor* proc);
+    Tree(const Str&, Bool);
     ~Tree();
     BOOL XSLTree;
-    eFlag appendVertex(Vertex *);
+    eFlag appendVertex(Sit S, Vertex *);
     Vertex *popVertex();
-    Vertex *appendText(char *, int);
-    eFlag parseFinished();
-    void output();
+    Vertex *appendText(Sit S, char *, int);
+    eFlag parseFinished(Sit S);
     void dump();
-    void
-        setName(const Str&);
-    RootNode root;
-    DataLine *dline;
-    // Box treeBox;
+    void setName(const Str&);
     Vertex *stackTop;
     Str name;
     NSList pendingNS;
-    int
-        stripped;
+    int stripped;
     void dropCurrentElement(Vertex *);
-    OutputDefinition
-        outputDef;
+    OutputDefinition outputDef;
     void flushPendingText();
-    Processor* proc;
+    Element& dummyElement() const;
+    HashTable& dict();
+    Arena& getArena();
+    const QName& getTheEmptyQName() const;
+    Phrase stdPhrase(StandardPhrase phrase_) const 
+    { 
+        assert(phrase_ <= PHRASE_LAST); 
+	    return stdPhrases[phrase_]; 
+	};
+	Bool cmpQNames(const QName &q1, const QName &q2) const;
+	Bool cmpQNamesForeign(const QName &q, const HashTable& dictForeign, const QName &qForeign);
+	void expandQ(const QName& q, EQName &expanded);
+	void expandQStr(const QName& q, Str &expName);
+	const Str& expand(Phrase ph);
+	Phrase unexpand(const Str& strg);
+	RootNode& getRoot() {return *NZ(root);}
+	eFlag serialize(Sit S, char *& result);
+	void makeStamps();
+	RuleSList &rules();
+	P2List &aliases();
+    eFlag processVertexAfterParse(Sit S, Vertex *v, TreeConstructer* tc);
+    eFlag parse(Sit S, DataLine *d);    
 private:
+    Arena theArena;
     Text *pendingTextNode;
     DStr pendingText;
-    int
-        vcount;             // # of vertices in tree
+    int vcount;             // # of vertices in tree
+    Element *theDummyElement;
+    HashTable theDictionary;
+    const QName theEmptyQName;
+    Phrase stdPhrases[PHRASE_LAST];
+    void initDict();
+    RootNode *root;
+    RuleSList rulesList;
+    P2List aliasesList;
+    Element* findStylesheet(Daddy& d);
+    eFlag insertRule(Sit S, XSLElement *tmpl);
+    double defaultPriorityLP(Expression *);
+    double defaultPriority(XSLElement *);      
+    void report(Sit S, MsgType type, MsgCode code, 
+        const Str &arg1, const Str &arg2) const;
 };
 
 #endif //ifndef TreeHIncl
