@@ -38,6 +38,8 @@
 #include <string.h>
 #include "proc.h"
 
+// GP: clean
+
 /*****************************************************************
 
     global functions
@@ -352,10 +354,8 @@ DataLine::DataLine()
   This constructor just sets everything to zeroes and such.
 *****************************************************************/
 
-DataLine::DataLine(Processor* proc_)
+DataLine::DataLine()
 {
-    proc = proc_;
-    situation = proc->situation;
     mode = DLMODE_NONE;       
     scheme = URI_NONE;      
     f = NULL;               
@@ -378,8 +378,9 @@ DataLine::~DataLine()
 
 DataLine::~DataLine()
 {
-    assert(mode == DLMODE_CLOSED);
-    assert(!f);
+    // removing the asserts (can be killed anytime due to error)
+    // assert(mode == DLMODE_CLOSED || mode == DLMODE_NONE);
+    // assert(!f);
     // if there is an outBuf, delete it now
     if (outBuf)
         delete outBuf;
@@ -401,17 +402,17 @@ _baseUri    the base URI used in case the reference in _uri is
 _mode       the access mode (DLMODE_READ, DLMODE_WRITE)
 *****************************************************************/
 
-eFlag DataLine::open(const char *_uri, DLAccessMode _mode)
+eFlag DataLine::open(Sit S, const char *_uri, DLAccessMode _mode, StrStrList* argList_)
 {
     assert(mode == DLMODE_NONE);  // the buffer must not be open yet
-    mode = _mode;
-    fullUri = (char*)_uri;
-
     // combine _uri and _baseUri into one
     Str strScheme, strPath;
     scheme = uri2SchemePath(_uri, strScheme, strPath);
-
     char *name = (char*) strPath;
+
+    // mode set in the end
+    fullUri = (char*)_uri;
+
     switch(scheme)
     {
     case URI_FILE:
@@ -419,9 +420,12 @@ eFlag DataLine::open(const char *_uri, DLAccessMode _mode)
             if (name[0] == '/' && name[1] == '/')
                 name += 2;          // skipping the "//" in front
             // try to open the file
-            if (!(f = stdopen(name,
-                _mode == DLMODE_WRITE ? "wt" : "rt")))
-                Err1(situation, E_FILE_OPEN, name);
+#ifdef _MSC_VER
+            if (!(f = stdopen(name,_mode == DLMODE_WRITE ? "wb" : "rt")))
+#else
+            if (!(f = stdopen(name,_mode == DLMODE_WRITE ? "w" : "r")))
+#endif
+                Err1(S, E_FILE_OPEN, name);
             // set fileIsStd if filename is "stdin", "stdout" or "stderr"
             fileIsStd = isstd(name);
         }; break;
@@ -432,12 +436,12 @@ eFlag DataLine::open(const char *_uri, DLAccessMode _mode)
             if (_mode == DLMODE_READ)
             {
                 Bool isUTF16 = FALSE;
-                /* 
-                  call getArg to set:
-                  'buffer' to the text block pointer
-                  'isUTF16' to TRUE if the arg is UTF-16 encoded
-                */
-                E( proc -> getArg(name, buffer, isUTF16) );
+		        Str *value = NULL;
+			    if (argList_)
+			        value = argList_ -> find(name);
+			    if (!value)
+                    Err1(S, E1_ARG_NOT_FOUND, name);
+		        buffer = (char*)*value;
             }
             // if opening for write access, just allocate a new dynamic block
             else
@@ -447,10 +451,14 @@ eFlag DataLine::open(const char *_uri, DLAccessMode _mode)
         {
             // try the extending scheme handler
             // ask the handler address from the Processor
-            handler = proc -> getSchemeHandler(&handlerUD);
+	        Processor *proc = S.getProcessor();
+		    if (proc)
+                handler = proc->getSchemeHandler(&handlerUD);
+		    else
+		        handler = NULL;
             // if there is no handler, report unsupported scheme
             if (!handler)
-                Err1(situation, E1_UNSUPPORTED_SCHEME, strScheme);
+                Err1(S, E1_UNSUPPORTED_SCHEME, strScheme);
             // try the fast way
             int count;
             buffer = NULL;
@@ -469,9 +477,9 @@ eFlag DataLine::open(const char *_uri, DLAccessMode _mode)
                     strScheme, name, &handle))
                 {
                 case SH_ERR_UNSUPPORTED_SCHEME:     // scheme not supported
-                    Err1(situation, E1_UNSUPPORTED_SCHEME, strScheme);
+                    Err1(S, E1_UNSUPPORTED_SCHEME, strScheme);
                 case SH_ERR_NOT_OK:
-                    Err1(situation, E1_URI_OPEN, strScheme + ":" + strPath);
+                    Err1(S, E1_URI_OPEN, strScheme + ":" + strPath);
                 };
             }
         };
@@ -486,7 +494,7 @@ DataLine::close()
 
   closes the resource attached to this data line.
 *****************************************************************/
-eFlag DataLine::close()
+eFlag DataLine::close(Sit S)
 {
     assert(mode != DLMODE_NONE);
     switch(scheme)
@@ -497,7 +505,7 @@ eFlag DataLine::close()
             if (!fileIsStd)
             {
                 if (fclose(f))
-                    Err1(situation, E1_URI_CLOSE, fullUri);
+                    Err1(S, E1_URI_CLOSE, fullUri);
             };
             f = NULL;
         }; break;
@@ -507,12 +515,12 @@ eFlag DataLine::close()
         {
             if (gotWholeDocument)
             {
-                NZ(handler) -> freeMemory(handlerUD, proc, buffer);
+                NZ(handler) -> freeMemory(handlerUD, S.getProcessor(), buffer);
             }
             else
             {
-                if(NZ(handler) -> close(handlerUD, proc, handle))
-                    Err1(situation, E1_URI_CLOSE, fullUri);
+                if(NZ(handler) -> close(handlerUD, S.getProcessor(), handle))
+                    Err1(S, E1_URI_CLOSE, fullUri);
             }
         }; break;
     };
@@ -535,12 +543,9 @@ int my_wcslen(const char *p)
     return len;
 }
 
-eFlag DataLine::save(const char *data, int length)
+eFlag DataLine::save(Sit S, const char *data, int length)
 {
     assert(mode == DLMODE_WRITE); // assume the file open for writing
-    // PRB: data should be output one character at a time, so far we are
-    // outputting in bulk
-    //
     // int length = utf16Encoded ? my_wcslen(data) : strlen(data);
     switch (scheme)             // choose the output procedure 
     {
@@ -558,8 +563,8 @@ eFlag DataLine::save(const char *data, int length)
     case URI_EXTENSION:         // external handler
         {
             int actual = length;
-            if( NZ(handler) -> put(handlerUD, proc, handle, data, &actual) )
-                Err1(situation, E1_URI_WRITE, fullUri);
+            if( NZ(handler) -> put(handlerUD, S.getProcessor(), handle, data, &actual) )
+                Err1(S, E1_URI_WRITE, fullUri);
         };
     }
     return OK;
@@ -587,7 +592,7 @@ is16    TRUE iff the string is UTF-16
 - if a terminating 0 is reached, copying stops
 *****************************************************************/
 
-int DataLine::get(char *dest,int maxcount)
+int DataLine::get(Sit S, char *dest,int maxcount)
 {
     int result;
     assert(mode == DLMODE_READ);  // assume the file open for reading
@@ -631,8 +636,11 @@ int DataLine::get(char *dest,int maxcount)
             else
             {
                 int actual = maxcount;
-                if( NZ(handler) -> get(handlerUD, proc, handle, dest, &actual) )
-                    Err1(situation, E1_URI_READ, fullUri);
+                if( NZ(handler) -> get(handlerUD, S.getProcessor(), handle, dest, &actual) )
+                {
+                    S.message( MT_ERROR, E1_URI_READ, fullUri, "" );
+                    return -1;
+                }
                 result = actual;
             }
         }; break;
@@ -654,3 +662,18 @@ DynBlock* DataLine::getOutBuffer()
     assert(mode == DLMODE_WRITE && scheme == URI_ARG); 
     return NZ(outBuf); // -> getPointer();
 }
+
+eFlag DataLine::setURIAndClose(Sit S, const char *_uri)
+{
+    assert( mode == DLMODE_NONE );
+    mode = DLMODE_CLOSED;
+    scheme = URI_ARG;
+    fullUri = _uri;
+    return OK;
+}
+
+void DataLine::report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2)
+{
+    S.message(type, code, arg1, arg2);
+}
+
