@@ -35,17 +35,7 @@
 //      utf8.cpp
 //
 
-/*  
- *  if the compiler has __restrict, it may cause problems in calls to iconv
- *  see utf8RecodeTo
- *  compare <sys/cdefs.h>
- */
-
-/*
-#if !defined __GNUC__ || __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 92)
-#define UTF8_ICONV_CAST_OK 1
-#endif
-*/
+// GP: clean
 
 #ifdef HAVE_CONFIG_H
 #include <autocfg.h>
@@ -54,27 +44,10 @@
 #include <assert.h>
 #include "utf8.h"
 
-#ifdef HAVE_ICONV_H
-#include <iconv.h>
-#endif
-
 #if defined(HAVE_WCHAR_H) || defined(_MSC_VER)
 #include <wchar.h>
 #endif
 
-/*  This MUST match the Encoding enum defined in utf8.h  */
-static const char* iconv_encoding[8] =
-{
-    "UTF8",
-    "UTF16",
-    "ASCII",
-    "ISO-8859-1",
-    "ISO-8859-2",
-    "CP1250",
-    "EUC-JP",
-    "SHIFT-JIS"
-};
-
 int utf8SingleCharLength (const char* text)
 {
   if (!(*text & 0x80)) return 1;
@@ -100,7 +73,7 @@ unsigned long utf8CharCode(const char *text)
 
 unsigned long utf16CharCode(const wchar_t *src)
 {
-    unsigned long first = *src;
+  unsigned long first = *src & 0xffffUL; // fix for Solaris
     if (first < 0xd800U || first >= 0xe000U)
         return first;
     unsigned short second = (unsigned short)(src[1]);
@@ -112,7 +85,9 @@ unsigned long utf16CharCode(const wchar_t *src)
 // returns length in *words*
 int utf16SingleCharLength(const wchar_t *src)
 {
-    wchar_t first = *src;
+    // issues a warning on win32:
+    // wchar_t first = *src & 0xffffUL;
+    wchar_t first = *src & -1; // fix for Solaris
     if (first < 0xd800U || first >= 0xe000U)
         return 1;
     else
@@ -232,65 +207,8 @@ int utf8GetChar(char *dest, const char *src)
     return len;
 }
 
-Bool utf8CanRecodeTo(const char *destEncoding)
-{
-    if (strEqNoCase(destEncoding, "UTF8") ||
-        strEqNoCase(destEncoding, "UTF-8"))
-        return TRUE;
-    // more checks for internally supported encodings can come here
-#if defined(HAVE_ICONV_H)
-    iconv_t cd = iconv_open (destEncoding, "UTF-8");
-    if (cd != (iconv_t)(-1))
-    {
-        iconv_close (cd);
-        return TRUE;
-    }
-#endif
-    return FALSE;
-}
 
-int utf8InternalRecode(char *dest, const char *src, Encoding enc)
-{
-    if (enc == ENC_UTF8)
-        return utf8GetChar(dest, src);
-    else
-        return 0;
-}
-
-int utf8Recode(char* dest, const char* src, Encoding enc)
-{
-    int internal = utf8InternalRecode(dest, src, enc);
-    if (internal)
-        return internal;
-#if !defined(HAVE_ICONV_H)
-    return 0;
-#else
-    iconv_t  cd;
-    size_t   inbytesleft  = utf8SingleCharLength (src);
-    size_t   outbytesleft = SMALL_BUFFER_SIZE;
-    char    *outbuf       = dest;
-    
-    cd = iconv_open (iconv_encoding[enc], "UTF-8");
-    assert(cd != (iconv_t)(-1));
-    // --tom Nov 10, 2000
-    // cast of &src to char** is necessary because of problems on Redhat 7
-    // but then egcs complains because of __restrict in iconv's declaration
-    // so we cast any time __restrict isn't known (ie. when not using egcs >= 1.2)
-    // but RH7 problem probably persists, will have to fork some more if so
-    while (inbytesleft && 
-        iconv(cd, 
-#ifdef SABLOT_ICONV_CAST_OK
-        (char**)(&src),
-#else
-        &src, 
-#endif
-        &inbytesleft, &outbuf, &outbytesleft) != -1);
-    iconv_close (cd);
-    return SMALL_BUFFER_SIZE - outbytesleft;
-#endif
-}
-
-// tests wether a character in "c" is Within Range
+// tests whether a character in "c" is Within Range
 #define wr(a,b) (((c) - (a)) <= (b) - (a))
 
 Bool utf8IsDigit(unsigned long c)
