@@ -33,6 +33,8 @@
 #ifndef VarsHIncl
 #define VarsHIncl
 
+// GP: clean
+
 #include "base.h"
 #include "datastr.h"
 #include "expr.h"
@@ -49,25 +51,27 @@ public:
 class VarBindings /* : public SabObj */
 {
 public:
-    VarBindings(Processor *proc_) : proc (proc_), varname (proc_)
+    VarBindings(QName& varname_)
+    : varname(varname_)
     {
     }
     ~VarBindings();
     QName varname;
     Expression *global;
     PList <VarBindingItem*> bindings;
-    Processor *proc;
 };
 
+class Tree;
+
 class VarsList : public SList<VarBindings *>
 {
 public:
-    VarsList(Processor* proc);
+    VarsList(Tree&);
     ~VarsList();
     VarBindings* find(QName&);
     virtual int compare(int first, int second);
-    eFlag addBinding(QName&, Expression *, Bool force);
-    eFlag addPrebinding(QName&, Expression *);
+    eFlag addBinding(Sit S, QName&, Expression *, Bool force);
+    eFlag addPrebinding(Sit S, QName&, Expression *);
     void rmBinding(QName&);
     Expression *getBinding(QName &);
     void startCall();
@@ -79,10 +83,10 @@ public:
     void rmPrebindings();
     int currCallLevel, currNestLevel;
 private:
-    Processor* proc;
-    SituationObj* situation;
     VarBindings *getOrAdd(QName &);
     void _endCall(Bool rmLastLevelPrebs);
+    void report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2);
+    Tree& sheet;
 };
 
 #endif  // ifndef VarsHIncl
