@@ -33,16 +33,20 @@
 #ifndef VertsHIncl
 #define VertsHIncl
 
+// GP: clean
 
 #include "base.h"
 #include "arena.h"
 #include "datastr.h"
 #include "expr.h"
 #include "hash.h"
+// #include "tree.h"
+
+class Tree;
 
 class Context;
 class Expression;
-
+class OutputterObj;
 class AttList;
 
 /****************************************
@@ -52,30 +56,42 @@ V e r t e x
 class Vertex : public ArenaMember
 {
 public:
-    Vertex(Processor* proc_, VTYPE avt = VT_VERTEX_WF)
-        : vt(avt), parent(NULL), ordinal(0), proc(proc_)
+    Vertex(Tree& owner_, VTYPE avt = VT_VERTEX_WF)
+        : owner(owner_), vt(avt), parent(NULL), ordinal(0)
     {
         stamp = lineno = 0;
+	    instanceData = NULL;
     };
     virtual ~Vertex();
-    virtual eFlag execute(Context *);
-   	virtual eFlag value(DStr &, Context *);
-    virtual eFlag startCopy();
-    virtual eFlag endCopy();
-    virtual eFlag copy();
-    void old__output(DataLine &);
+    virtual eFlag execute(Sit S, Context *);
+   	virtual eFlag value(Sit S, DStr &, Context *);
+    virtual eFlag startCopy(Sit S, OutputterObj& out);
+    virtual eFlag endCopy(Sit S, OutputterObj& out);
+    virtual eFlag copy(Sit S, OutputterObj& out);
     virtual void speak(DStr &, SpeakMode);
     void setParent(Vertex *v);
+    Vertex *getPreviousSibling();
+    Vertex *getNextSibling();
     virtual const QName& getName() const;
+    Tree& getOwner() const;
+    virtual eFlag serialize(Sit S, OutputterObj &out);
+
     Vertex *parent;
-    Processor* proc;
     int 
         ordinal,
         lineno;
     VTYPE vt;
     int 
         stamp;          // the vertex number in doc order (0 = root)
-
+    void report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2) const;
+    HashTable& dict() const;
+    void setInstanceData(void *data) {instanceData = data;}
+    void* getInstanceData() {return instanceData;}
+    virtual void makeStamps(int& stamp);
+protected:
+    Tree &owner;
+private:
+    void *instanceData;    // for Perl interface
 };
 
 /****************************************
@@ -89,14 +105,18 @@ class VertexList: public SList<Vertex*>
 public:
     VertexList(int logBlockSize_ = LIST_SIZE_SMALL);
     virtual ~VertexList();
-    eFlag execute (Context*);
-    eFlag value(DStr&, Context *);
+    eFlag execute (Sit S, Context*);
+    eFlag value(Sit S, DStr&, Context *);
     virtual void speak(DStr &, SpeakMode);
     virtual void append(Vertex *);
-    void appendAndSetOrdinal(Vertex *v);
+    void rm(int ndx);
     void destructMembers();
     int strip();
-    eFlag copy();
+    eFlag copy(Sit S, OutputterObj& out);
+    void insertBefore(Vertex *newChild, int refIndex);
+    int getIndex(Vertex *v);
+    eFlag serialize(Sit S, OutputterObj &out);
+    void makeStamps(int& stamp);
 };
 
 /****************************************
@@ -108,7 +128,8 @@ class ArenaVertexList : public VertexList, public ArenaMember
 public:
     ArenaVertexList(Arena *arena__=NULL, int logBlockSize_ = LIST_SIZE_SMALL)
         : VertexList(logBlockSize_), arena_(arena__) 
-    {};
+    {
+    };
 
     ~ArenaVertexList()
     {
@@ -155,39 +176,18 @@ D a d d y
 class Daddy : public Vertex
 {
 public:
-    Daddy(Processor* proc, VTYPE avt = VT_DADDY_WF)
-        : Vertex(proc, avt), contents(proc -> getArena())
-    {};
+    Daddy(Tree& owner_, VTYPE avt = VT_DADDY_WF);
     virtual ~Daddy(void);
-    virtual eFlag execute(Context *c);
-   	virtual eFlag value(DStr &, Context *);
-    virtual eFlag newChild(Vertex*);
-    virtual eFlag checkChildren();
+    virtual eFlag execute(Sit S, Context *c);
+   	virtual eFlag value(Sit S, DStr &, Context *);
+    virtual eFlag newChild(Sit S, Vertex*);
+    virtual eFlag checkChildren(Sit S);
     virtual void speak(DStr &, SpeakMode);
     virtual int strip();
     //
     ArenaVertexList contents;
 };
 
-/****************************************
-R o o t N o d e
-****************************************/
-
-class RootNode : public Daddy
-{
-public:
-    RootNode(Processor* proc_)
-        : Daddy(proc_, VT_ROOT_WF)
-    {
-    };
-    virtual ~RootNode();
-    virtual eFlag execute(Context *c);
-    virtual eFlag newChild(Vertex*);
-    virtual eFlag checkChildren();
-    virtual void speak(DStr &, SpeakMode);
-    virtual eFlag copy();
-};
-
 /****************************************
 A t t r i b u t e
 ****************************************/
@@ -195,14 +195,16 @@ A t t r i b u t e
 class Attribute : public Vertex
 {
 public:
-    Attribute(const QName&, const Str&, XSL_ATT, Processor* proc_);
+    Attribute(Tree& owner_, const QName&, const Str&, XSL_ATT);
     virtual ~Attribute();
-    eFlag buildExpr(Bool, ExType);
-    virtual eFlag execute(Context *c);
+    eFlag buildExpr(Sit S, Bool, ExType);
+    virtual eFlag execute(Sit S, Context *c);
     virtual void speak(DStr &, SpeakMode);
-   	virtual eFlag value(DStr &, Context *);
-    virtual eFlag startCopy();
+   	virtual eFlag value(Sit S, DStr &, Context *);
+    virtual eFlag startCopy(Sit S, OutputterObj& out);
     virtual const QName& getName() const;
+    void setValue(const Str& newValue);
+    virtual eFlag serialize(Sit S, OutputterObj &out);
     QName name;
     ArenaStr cont;
     Expression *expr;
@@ -220,6 +222,9 @@ public:
         : ArenaVertexList(arena_)
     {};
     Attribute *find(XSL_ATT);
+    // assumes that attName belongs to the same hash
+    Attribute *find(const QName& attName);
+    int findNdx(const QName& attName);
 };
 
 /*****************************************************************
@@ -229,15 +234,18 @@ public:
 class NmSpace: public Vertex
 {
 public:
-    NmSpace(Phrase prefix_, Phrase uri_, Processor* proc_);
+    NmSpace(Tree& owner_, Phrase prefix_, Phrase uri_);
     virtual ~NmSpace();
     Phrase
         prefix,
         uri;
-    virtual eFlag execute(Context *);
+	QName name;
+    virtual eFlag execute(Sit S, Context *);
     virtual void speak(DStr &, SpeakMode);
-   	virtual eFlag value(DStr &, Context *);
-    virtual eFlag startCopy();
+   	virtual eFlag value(Sit S, DStr &, Context *);
+    virtual eFlag startCopy(Sit S, OutputterObj& out);
+    virtual const QName& getName() const;
+    virtual eFlag serialize(Sit S, OutputterObj &out);
 };
 
 /*****************************************************************
@@ -247,16 +255,20 @@ public:
 class NSList : public ArenaVertexList
 {
 public:
-    NSList (Processor* proc_, Arena* arena_ = NULL)
-        : proc ( proc_ ), ArenaVertexList(arena_,0)
+    NSList (Arena* arena_ = NULL)
+        : ArenaVertexList(arena_,0)
     { }
 
     ~NSList();
     NmSpace *find(Phrase) const;
-    eFlag resolve(Phrase&, Bool) const;
+    eFlag resolve(Sit S, Phrase&, Bool) const;
     void unresolve(Phrase&) const;
-    void giveCurrent(NSList &, Tree*) const;
-    Processor* proc;
+    void giveCurrent(Sit S, NSList &, Tree*) const;
+
+    // unresolve the qname's prefix
+    void findPrefix(QName &q);
+    void report(Sit S, MsgType type, MsgCode code, 
+        const Str &arg1, const Str &arg2) const;
 };
 
 /****************************************
@@ -266,27 +278,52 @@ E l e m e n t
 class Element : public Daddy
 {
 public:
-    Element(QName&, Tree *_ownerT, Processor* proc_, VTYPE = VT_ELEMENT_WF);
+    Element(Tree& owner_, QName&, VTYPE = VT_ELEMENT_WF);
     virtual ~Element();
-    virtual eFlag execute(Context *c);
-    virtual eFlag newChild(Vertex*);
+    virtual eFlag execute(Sit S, Context *c);
+    virtual eFlag newChild(Sit S, Vertex*);
     virtual void speak(DStr &, SpeakMode);
-    void removeBindings();
-    virtual eFlag startCopy();
-    virtual eFlag endCopy();
-    virtual eFlag copy();
+    void removeBindings(Sit S);
+    virtual eFlag startCopy(Sit S, OutputterObj& out);
+    virtual eFlag endCopy(Sit S, OutputterObj& out);
+    virtual eFlag copy(Sit S, OutputterObj& out);
     virtual const QName& getName() const;
+    virtual eFlag serialize(Sit S, OutputterObj &out);
+    void removeChild(Vertex *child);
+    virtual void makeStamps(int& stamp);
+    
+    // set qname to "prefix:local". Last param says
+    // whether to expand the default namespace
+    eFlag setLogical(Sit S, QName &q, const Str&, Bool defaultToo) const;
     //
     NSList
         namespaces;
     AttList 
         atts;
-    Tree
-        *ownerT;
     QName
         name;
 };
 
+/****************************************
+R o o t N o d e
+****************************************/
+
+class RootNode : public Element
+{
+public:
+    RootNode(Tree& owner_, QName& name_)
+        : Element(owner_, name_, VT_ROOT_WF)
+    {
+    };
+    virtual ~RootNode();
+    virtual eFlag execute(Sit S, Context *c);
+    virtual eFlag newChild(Sit S, Vertex*);
+    virtual eFlag checkChildren(Sit S);
+    virtual void speak(DStr &, SpeakMode);
+    virtual eFlag copy(Sit S, OutputterObj& out);
+    virtual eFlag serialize(Sit S, OutputterObj &out);
+};
+
 /****************************************
 T e x t
 ****************************************/
@@ -295,18 +332,17 @@ class Text : public Vertex
 {
 public:
     ArenaStr cont;
-    Text(char *acont, Processor* proc_, Arena* arena_, int alen=0)
-        : Vertex(proc_, VT_TEXT_WF), cont(arena_)
-    {
-        if (alen)
-            cont.nset(acont,alen);
-        else cont = (char*) acont;
-    };
+    Text(Tree& owner_, char *acont, int alen=0);
     virtual ~Text();
-    virtual eFlag execute(Context *);
+    virtual eFlag execute(Sit S, Context *);
     virtual void speak(DStr &, SpeakMode);
-   	virtual eFlag value(DStr &, Context *);
-    virtual eFlag startCopy();
+   	virtual eFlag value(Sit S, DStr &, Context *);
+    virtual eFlag startCopy(Sit S, OutputterObj& out);
+    virtual eFlag serialize(Sit S, OutputterObj &out);
+    void beCDATA();
+    Bool isCDATA(); 
+private:
+    Bool isCDATAFlag;
 };
 
 
@@ -317,13 +353,14 @@ C o m m e n t
 class Comment: public Vertex
 {
 public:
-    ArenaStr cont;
-    Comment(const Str& cont_, Processor* proc_);
+    Comment(Tree& owner_, const Str& cont_);
     virtual ~Comment();
-    virtual eFlag execute(Context *);
+    virtual eFlag execute(Sit S, Context *);
     virtual void speak(DStr &, SpeakMode);
-   	virtual eFlag value(DStr &, Context *);
-    virtual eFlag startCopy();
+   	virtual eFlag value(Sit S, DStr &, Context *);
+    virtual eFlag startCopy(Sit S, OutputterObj& out);
+    virtual eFlag serialize(Sit S, OutputterObj &out);
+    ArenaStr cont;
 };
 
 
@@ -334,13 +371,14 @@ P r o c I n s t r
 class ProcInstr: public Vertex
 {
 public:
-    ProcInstr(Phrase name_, const Str& cont_, Processor* proc_);
+    ProcInstr(Tree& owner_, Phrase name_, const Str& cont_);
     virtual ~ProcInstr();
-    virtual eFlag execute(Context *);
+    virtual eFlag execute(Sit S, Context *);
     virtual void speak(DStr &, SpeakMode);
-   	virtual eFlag value(DStr &, Context *);
-    virtual eFlag startCopy();
+   	virtual eFlag value(Sit S, DStr &, Context *);
+    virtual eFlag startCopy(Sit S, OutputterObj& out);
     virtual const QName& getName() const;
+    virtual eFlag serialize(Sit S, OutputterObj &out);
     ArenaStr cont;
     QName name;
 };
@@ -353,22 +391,22 @@ X S L E l e m e n t
 class XSLElement : public Element
 {
 public:
-	eFlag checkAtts();       
-    XSLElement(QName&, Tree *_ownerT, XSL_OP, Processor* proc_);
-    virtual eFlag execute(Context *c);
-    virtual eFlag newChild(Vertex*);
-    eFlag checkToplevel();
-    virtual eFlag checkChildren();
+    XSLElement(Tree& owner_, QName&, XSL_OP);
+    virtual eFlag execute(Sit S, Context *c);
+    virtual eFlag newChild(Sit S, Vertex*);
+    eFlag checkToplevel(Sit S);
+    virtual eFlag checkChildren(Sit S);
     void checkExtraChildren(int& k);
     Expression *getAttExpr(XSL_ATT);
     virtual int strip();
     XSL_OP op;
+	eFlag checkAtts(Sit S);       
 //    VertexList defs;
 private:
     // applies only to XSL_APPLY_TEMPLATES and XSL_FOR_EACH:
-    eFlag makeSortDefs(SortDefList &sortDefs, Context *c);
+    eFlag makeSortDefs(Sit S, SortDefList &sortDefs, Context *c);
     // applies only to XSL_SORT:
-    eFlag make1SortDef(SortDef *&def, Context *c);
+    eFlag make1SortDef(Sit S, SortDef *&def, Context *c);
 };
 
 #endif //ifndef VertsHIncl
